﻿<%@ Page Language="C#" MasterPageFile="~/Default.master" ClassName="Elsinore.ScreenConnect.AdministrationPage" %>

<script runat="server">

	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);

		Permissions.AssertPermission(PermissionInfo.AdministerPermission, false);

		var tabIndex = Extensions.TryParseInt64(this.Context.Request.QueryString["Tab"]);
		var tabKeys = new[] { "Status", "Security", "Database", "Mail", "Triggers", "License", "Appearance", "Audit" };
		var tabNames = tabKeys.Select(k => Resources.Default.ResourceManager.GetString("AdministrationPanel." + k + "TabName")).ToArray();

		tabNames.Select((tn, i) => new HyperLink
		{
			NavigateUrl = "?Tab=" + i,
			Text = tn,
			CssClass = (i == tabIndex ? "Selected" : string.Empty)
		}).ForEach(h => this.tabPanel.Controls.Add(h));

		this.updatePanel.ContentTemplateContainer.Controls.Add(this.LoadControl("~/" + tabKeys[tabIndex] + ".ascx"));

		this.saveButton.Text = string.Format(Resources.Default.AdministrationPanel_SaveButtonTextFormat, tabNames[tabIndex]);

		this.AddJavascriptObjectDeclaration("window._administrationContext", new
		{
			referenceMessage = Resources.Default.ResourceManager.GetString("AdministrationPanel." + tabKeys[tabIndex] + "ReferenceMessage"),
			isSaveAllowed = this.GetAllControls().OfType<IDataControl>().Any(dc => dc.IsSaveAllowed),
			showReferenceText = DR.AdministrationPanel_ShowReferenceText,
			hideReferenceText = DR.AdministrationPanel_HideReferenceText,
		});
	}

	protected override void OnLoad(EventArgs e)
	{
		base.OnLoad(e);

		if (!this.IsPostBack)
		{
			var dataControl = this.GetAllControls().OfType<IDataControl>().FirstOrDefault();

			if (dataControl != null)
			{
				dataControl.Refresh();
				WebExtensions.RegisterUpdatePanelStartupScript(this, "Sys.WebForms.PageRequestManager.getInstance().add_beginRequest(SC.ui.setChangesMade);");
				WebExtensions.RegisterUpdatePanelStartupScript(this, "SC.ui.addHandler(SC.ui.get('detailPanel'), 'keydown', SC.ui.setChangesMade);");
			}
		}
	}

	protected override bool OnBubbleEvent(object source, EventArgs e)
	{
		if (e.As<CommandEventArgs>().SafeNav(c => c.CommandName) == "Save")
		{
			if (this.IsValid)
			{
				Proc proc = delegate
				{
					var dataControl = this.GetAllControls().OfType<IDataControl>().First();
					dataControl.Save();
					dataControl.Refresh();
				};

				if (WebExtensions.ExecuteWithResult(this.saveLabel, DR.AdministrationPanel_SaveMessageText, proc))
					WebExtensions.RegisterUpdatePanelStartupScript(this, "SC.ui.clearChangesMade();");
			}
		}

		return base.OnBubbleEvent(source, e);
	}

</script>
<asp:Content runat="server" ContentPlaceHolderID="Main">
	<form runat="server" id="form">
	<asp:ScriptManager runat="server" EnablePartialRendering="true" />
	<div id="masterPanel">
		<div id="masterSelectionPanel">
			<div>
				<h2><%= DR.AdministrationPanel_Title %></h2>
				<div runat="server" id="tabPanel" />
			</div>
		</div>
	</div>
	<div id="detailPanel">
		<div id="administrationPanel" class="AdministrationPanel">
			<div class="EditPanel">
				<asp:UpdatePanel runat="server" ID="updatePanel" UpdateMode="Conditional" />
			</div>
			<div class="ReferencePanel">
				<p id="referenceParagraph" />
			</div>
			<div class="ButtonPanel">
				<a href="#" id="referenceButton"></a>
				<asp:Button runat="server" ID="saveButton" CommandName="Save" />
				<asp:LoadingImage runat="server" CssClass="SmallLoadingImage" ImageUrl="Images/ActivityIndicator.gif" PostBackControl="saveButton" />
				<asp:Label runat="server" ID="saveLabel" />
			</div>
		</div>
	</div>
	</form>
</asp:Content>
<asp:Content runat="server" ContentPlaceHolderID="RunScriptBeforeRender">
	<script type="text/javascript">
		var administrationPanel = SC.ui.get('administrationPanel');
		SC.ui.ensureCssClass(administrationPanel, 'Button', window._administrationContext.isSaveAllowed);
		SC.ui.ensureCssClass(administrationPanel, 'ReferenceAvailable', !SC.util.isNullOrEmpty(window._administrationContext.referenceMessage));

		SC.ui.setElementContent(SC.ui.get('referenceParagraph'), window._administrationContext.referenceMessage, true);

		var referenceButton = SC.ui.get('referenceButton');
		referenceButton._commandName = 'ToggleReference';
		SC.ui.addCommandHandler(referenceButton);
		SC.ui.setElementContent(referenceButton, window._administrationContext.showReferenceText);

		window.document.documentElement._oncommand = function (command) {
			if (command.commandName == 'ToggleReference') {
				var isReferenceSet = SC.ui.toggleCssClass(SC.ui.get('administrationPanel'), 'Reference');
				SC.ui.setElementContent(SC.ui.get('referenceButton'), isReferenceSet ? window._administrationContext.hideReferenceText : window._administrationContext.showReferenceText);
			}
		};

		Sys.WebForms.PageRequestManager.getInstance().add_endRequest(function (sender, eventArgs) {
			var error = eventArgs.get_error();

			if (error != null) {
				eventArgs.set_errorHandled(true);

				if (error.httpStatusCode != 0) {
					// ASP.NET prepends the name, which is very unfriendly, to the description, with a colon and space after
					var message = error.message.substring(error.name.length + 2);
					SC.ui.showModalErrorBox(message);
				}
			}
		});
	</script>
</asp:Content>
