﻿<%@ Control Language="C#" EnableViewState="true" %>
<%@ Implements Interface="Elsinore.ScreenConnect.IDataControl" %>
<script runat="server">

	public void Refresh()
	{
		var configuration = Elsinore.ScreenConnect.WebConfigurationManager.OpenWebConfiguration();
		var maintenancePlanString = configuration.AppSettings.GetValue(ServerConstants.DatabaseMaintenancePlanSettingsKey);
		var maintenancePlan = MaintenancePlan.FromUserStringOrDefault(maintenancePlanString);

		this.daysBox.Text = maintenancePlan.RunEveryDayFrequency.ToString();
		this.timeBox.SelectedValue = TimeSpan.FromHours(maintenancePlan.RunAtLocalTimeHour).ToString();

		foreach (TableRow row in this.actionsTable.Rows)
		{
			var action = maintenancePlan.Actions.FirstOrDefault(c => c.DefinitionID == row.ID);
			var checkBox = row.GetAllControls().OfType<CheckBox>().FirstOrDefault();
			var textBoxes = row.GetAllControls().OfType<TextBox>();

			if (checkBox != null)
				checkBox.Checked = (action != null);

			foreach (var textBox in textBoxes)
			{
				textBox.Enabled = (action != null);

				var actionParameterValue = action.SafeNav(c => c.Parameters).SafeNav(ps => ps.TryGetValue(textBox.ID))
					?? MaintenancePlanActionDefinition.All.First(cd => cd.DefinitionID == row.ID).Parameters.First(p => p.ParameterDefinitionID == textBox.ID).DefaultValue;

				textBox.Text = Convert.ToString(actionParameterValue);
			}

			if (action != null && action.Parameters != null)
				foreach (var kvp in action.Parameters)
					((TextBox)row.FindControl(kvp.Key)).Text = Convert.ToString(kvp.Value);
		}
	}

	public void Save()
	{
		var maintenancePlan = new MaintenancePlan
		{
			RunEveryDayFrequency = Convert.ToByte(this.daysBox.Text),
			RunAtLocalTimeHour = (byte)TimeSpan.Parse(this.timeBox.Text).TotalHours,
			Actions = new List<MaintenancePlanAction>(),
		};

		foreach (TableRow row in this.actionsTable.Rows)
		{
			var checkBox = row.GetAllControls().OfType<CheckBox>().FirstOrDefault();

			if (checkBox != null && checkBox.Checked)
			{
				var actionDefinition = MaintenancePlanActionDefinition.All.First(cd => cd.DefinitionID == row.ID);

				maintenancePlan.Actions.Add(new MaintenancePlanAction
				{
					DefinitionID = row.ID,
					Parameters = row
						.GetAllControls().OfType<TextBox>()
						.Select(tb => Elsinore.ScreenConnect.Tuple.Create(tb.ID, tb.Text, actionDefinition.Parameters.First(p => p.ParameterDefinitionID == tb.ID).Type))
						.ToDictionary(t => t.Item1, t => Convert.ChangeType(t.Item2, t.Item3)),
				});
			}
		}

		var configuration = Elsinore.ScreenConnect.WebConfigurationManager.OpenWebConfiguration();
		configuration.AppSettings.SetValue(ServerConstants.DatabaseMaintenancePlanSettingsKey, MaintenancePlan.ToUserString(maintenancePlan));
		ServerToolkit.Instance.SaveConfiguration(configuration);
	}

	public bool IsSaveAllowed { get { return true; } }

	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);

		var maintenancePlanParameters = MaintenancePlanActionDefinition.All.SelectMany(c => c.Parameters).Distinct().ToArray();

		var headerRow = this.actionsTable.AddRow<TableRow>();
		headerRow.AddCell<TableHeaderCell>(null);
		headerRow.AddCell<TableHeaderCell>(DR.DatabasePanel_MaintenancePlanActionDescriptionText);

		foreach (var maintenancePlanParameter in maintenancePlanParameters)
			headerRow.Cells.Add(new TableHeaderCell { Text = DR.ResourceManager.GetString("DatabasePanel.MaintenancePlanParameterDescription." + maintenancePlanParameter.ParameterDefinitionID) });

		foreach (var maintenancePlanActionDefinition in MaintenancePlanActionDefinition.All)
		{
			var actionRow = this.actionsTable.AddRow<NamingContainerTableRow>();
			actionRow.ID = maintenancePlanActionDefinition.DefinitionID;

			var checkBox = new CheckBox { AutoPostBack = true };
			checkBox.CheckedChanged += this.OnCheckBoxCheckedChanged;

			actionRow.AddCell(checkBox);
			actionRow.AddCell(DR.ResourceManager.GetString("DatabasePanel.MaintenancePlanActionDescription." + maintenancePlanActionDefinition.DefinitionID));

			foreach (var maintenancePlanParameter in maintenancePlanParameters)
				actionRow.AddCell(maintenancePlanActionDefinition.Parameters.Contains(maintenancePlanParameter) ? new TextBox { ID = maintenancePlanParameter.ParameterDefinitionID, Text = Convert.ToString(maintenancePlanParameter.DefaultValue), Columns = 2 } : null);

			this.actionsTable.Rows.Add(actionRow);
		}

		for (var i = 0; i < TimeSpan.TicksPerDay / TimeSpan.TicksPerHour; i++)
			this.timeBox.Items.Add(TimeSpan.FromTicks(i * TimeSpan.TicksPerHour).ToString());
	}

	void OnCheckBoxCheckedChanged(object sender, EventArgs e)
	{
		var checkBox = (CheckBox)sender;
		var row = checkBox.FindAncestor<TableRow>();

		foreach (var textBox in row.GetAllControls().OfType<TextBox>())
			textBox.Enabled = checkBox.Checked;
	}

	class NamingContainerTableRow : TableRow, INamingContainer { }

</script>
<dl>
	<dt runat="server" innerhtml="<%$ Resources:Default, DatabasePanel.MaintenancePlanActionsText %>" />
	<dd>
		<asp:Table runat="server" ID="actionsTable" CssClass="DataTable" />
	</dd>
	<dt runat="server" innerhtml="<%$ Resources:Default, DatabasePanel.MaintenancePlanScheduleText %>" />
	<dd>
		<asp:Literal runat="server" Text="<%$ Resources:Default, DatabasePanel.MaintenancePlanScheduleDescription1 %>" />
		<asp:TextBox runat="server" ID="daysBox" Columns="2" />
		<asp:RequiredFieldValidator runat="server" ControlToValidate="daysBox" Text="*" Display="Dynamic" />
		<asp:RangeValidator runat="server" ControlToValidate="daysBox" MinimumValue="1" MaximumValue="9999" Text="*" Display="Dynamic" />
		<asp:Literal runat="server" Text="<%$ Resources:Default, DatabasePanel.MaintenancePlanScheduleDescription2 %>" />
		<asp:DropDownList runat="server" ID="timeBox" />
		<asp:Literal runat="server" Text="<%$ Resources:Default, DatabasePanel.MaintenancePlanScheduleDescription3 %>" />
	</dd>
</dl>
