﻿<%@ Page Language="C#" MasterPageFile="~/Default.master" %>

<%@ Import Namespace="System.Security.Principal" %>

<script runat="server">
	
	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);

		if (!this.IsPostBack)
		{
			this.multiView.ActiveViewIndex = 0;

			this.userNameBox.Text = Elsinore.ScreenConnect.FormsAuthenticationModule.GetUserName(this.Context.User);

			FormsAuthentication.SignOut();
			this.Context.User = new GenericPrincipal(new GenericIdentity(string.Empty), new string[0]);

			if (!AppSettingsCache.AllowPasswordAutoComplete)
				this.passwordBox.Attributes.Add("autocomplete", "off");

			this.instructionsLabel.Text = this.Request.QueryString[ServerConstants.LoginReasonParameterName]
				.TrySafeNav(s => (LoginReason)int.Parse(s))
				.SafeNav(lr => DR.ResourceManager.GetString("LoginPanel.LoginReason." + lr + ".Message"));
		}
	}

	protected override void OnPreRender(EventArgs e)
	{
		base.OnPreRender(e);

		if (!this.IsPostBack)
			this.userNameBox.If(t => t.Text == string.Empty, this.passwordBox).Focus();
	}

	void OnLoginButtonClick(object sender, EventArgs e)
	{
		var userName = this.userNameBox.Text.Trim();
		var password = this.ViewState["Password"].As<string>().IfNotNullOrEmpty(this.passwordBox.Text.Trim());
		var oneTimePassword = this.oneTimePasswordBox.Text.Trim();

		try
		{
			if (!Membership.ValidateUser(userName, password))
				throw new System.Security.SecurityException("Invalid credentials");

			var membershipUser = Membership.GetUser(userName).AssertNonNull();

			string oneTimePasswordUserKey, oneTimePasswordProviderKey;
			var oneTimePasswordProvider = default(OneTimePasswordAuthenticationProvider);

			if (OneTimePasswordAuthenticationProvider.ParsePasswordQuestion(membershipUser.PasswordQuestion, out oneTimePasswordUserKey, out oneTimePasswordProviderKey))
				oneTimePasswordProvider = OneTimePasswordAuthenticationProvider.GetInstance(oneTimePasswordProviderKey);

			if (this.multiView.ActiveViewIndex == 1)
			{
				if (!oneTimePasswordProvider.ValidateOneTimePassword(membershipUser.UserName, oneTimePasswordUserKey, oneTimePassword))
					throw new System.Security.SecurityException("One-time password invalid");

				this.OnLoginSuccess(userName);
			}
			else if (oneTimePasswordUserKey == null)
			{
				this.OnLoginSuccess(userName);
			}
			else
			{
				oneTimePasswordProvider.PrepareOneTimePassword(membershipUser.UserName, oneTimePasswordUserKey, Resources.Default.ResourceManager);

				this.ViewState["Password"] = password;
				this.multiView.ActiveViewIndex = 1;
				this.instructionsLabel.Text = DR.LoginPanel_OneTimePasswordMessage;
				this.oneTimePasswordBox.Focus();
			}
		}
		catch
		{
			this.loginErrorLabel.Visible = true;
		}
	}

	void OnLoginSuccess(string userName)
	{
		Membership.Provider.GetUser(userName)
			.SafeNav(u => Elsinore.ScreenConnect.FormsAuthenticationModule.CreateTicket(u))
			.SafeNav(t => Elsinore.ScreenConnect.FormsAuthenticationModule.CreateCookie(t))
			.SafeDo(c => this.Response.AppendCookie(c));

		var redirectUrl = this.Context.Request.QueryString[ServerConstants.LoginReturnUrlParameterName] ?? FormsAuthentication.DefaultUrl;
		this.Response.Redirect(redirectUrl);
	}
	
</script>
<asp:Content runat="server" ContentPlaceHolderID="Main">
	<div class="ContentPanel Login">
		<form runat="server">
		<h1 runat="server" innerhtml="<%$ Resources:Default, LoginPanel.Heading %>" />
		<p><asp:Label runat="server" ID="instructionsLabel" />
		<dl class="LoginPanel">
			<asp:MultiView runat="server" ID="multiView">
				<asp:View runat="server">
					<dt runat="server" innerhtml="<%$ Resources:Default, LoginPanel.UserNameLabelText %>" />
					<dd>
						<asp:TextBox runat="server" ID="userNameBox" /></dd>
					<dt runat="server" innerhtml="<%$ Resources:Default, LoginPanel.PasswordLabelText %>" />
					<dd>
						<asp:TextBox runat="server" ID="passwordBox" TextMode="Password" /></dd>
				</asp:View>
				<asp:View runat="server">
					<dt runat="server" innerhtml="<%$ Resources:Default, LoginPanel.OneTimePasswordLabelText %>" />
					<dd>
						<asp:TextBox runat="server" ID="oneTimePasswordBox" /></dd>
				</asp:View>
			</asp:MultiView>
			<dt></dt>
			<dd>
				<asp:Button runat="server" OnClick="OnLoginButtonClick" Text="<%$ Resources:Default, LoginPanel.ButtonText %>" />
				<asp:Label runat="server" ID="loginErrorLabel" EnableViewState="false" Visible="false" Text="<%$ Resources:Default, LoginPanel.FailureText %>" ForeColor="Red" />
			</dd>
		</dl>
		</form>
	</div>
</asp:Content>
