﻿window.SC = window.SC || {};
SC.launch = SC.launch || {};

SC.launch.getLaunchSuggestionHtml = function () {
	if (SC.util.isCapable(SC.util.Caps.Windows) && !SC.util.isCapable(SC.util.Caps.ClickOnce)) {
		if (SC.util.isCapable(SC.util.Caps.Firefox))
			return SC.res['Launcher.FirefoxClickOnceSuggestion'];
		else if (SC.util.isCapable(SC.util.Caps.Chrome))
			return SC.res['Launcher.ChromeClickOnceSuggestion'];
	}

	return null;
}

SC.launch.getJoinPanelDefinitionTreeRoot = function () {
	if (!window._joinPanelDefinitionTreeRoot) {
		var createPanelDefinitionFunc = function (key, parentSelectionHeadingText, parentSelectionButtonText, parentSelectionDescriptionText, childPanels, shouldAutoNavigateToFirstChild, shouldRememberSelection, joinProc, joinContentBuilderProc, connectedContentBuilderFunc) {
			return {
				key: key,
				parentSelectionHeadingText: parentSelectionHeadingText || key,
				parentSelectionButtonText: parentSelectionButtonText,
				parentSelectionDescriptionText: parentSelectionDescriptionText,
				childPanels: childPanels,
				shouldAutoNavigateToFirstChild: shouldAutoNavigateToFirstChild,
				shouldRememberSelection: shouldRememberSelection,
				joinProc: joinProc,
				joinContentBuilderProc: joinContentBuilderProc,
				connectedContentBuilderFunc: connectedContentBuilderFunc,
			};
		};

		var createBaseLauncherPanelDefinitionFunc = function (launcherKey, containsCapabilityFunc, parentSelectionHeadingText, shouldRememberSelection, timeoutMilliseconds, joinProc, joinContentBuilderProc) {
			return createPanelDefinitionFunc(
				launcherKey,
				parentSelectionHeadingText,
				SC.util.getResourceWithFallback('JoinPanel.Launcher{0}.ButtonText', launcherKey),
				SC.util.getResourceWithFallback('JoinPanel.Launcher{0}.Description', launcherKey),
				null,
				false,
				shouldRememberSelection,
				joinProc,
				function (container, childPanels, childStringPaths, nextPanelDefinition, nextPanelStringPath) {
					var joiningText = SC.util.getResourceWithFallback('JoinPanel.Launcher{0}.JoiningText', launcherKey);
					var joiningParagraph = SC.ui.addElement(container, 'P', null, null, joiningText);

					var joiningParagraphAnimateProc = function (counter) {
						if (counter == 0 || SC.ui.isInBody(joiningParagraph)) {
							var currentContent = SC.ui.getElementContent(joiningParagraph);
							var newDots = new Array((counter % 8) + 2).join('.');
							SC.ui.setElementContent(joiningParagraph, joiningText + newDots);

							if (counter < 100)
								window.setTimeout(function () { joiningParagraphAnimateProc(counter + 1); }, 400);
						}
					};

					joiningParagraphAnimateProc(0);

					joinContentBuilderProc(container, childPanels, childStringPaths, nextPanelDefinition, nextPanelStringPath);

					var nextParagraph = SC.ui.addElement(container, 'P', { className: 'Help' });

					if (!timeoutMilliseconds) {
						SC.ui.addElement(nextParagraph, 'SPAN', null, null, SC.res['JoinPanel.Launcher.NextTroubleText']);
						SC.ui.addNonBreakingSpace(nextParagraph);
						SC.ui.addElement(nextParagraph, 'A', { _commandName: 'Navigate', _commandArgument: nextPanelStringPath }, null, SC.res['JoinPanel.Launcher.NextTryText']);
						SC.ui.addNonBreakingSpace(nextParagraph);
						SC.ui.addElement(nextParagraph, 'SPAN', null, null, '(' + nextPanelDefinition.key + ')');
					} else {
						var initialLaunchTime = new Date();
						var blurred = false;
						var blurHandlerProc = function () { blurred = true; };

						var tickProc = function () {
							var millisecondsUntilTimeout = timeoutMilliseconds - (new Date() - initialLaunchTime);
							var done = false;

							if (blurred || document.webkitHidden || millisecondsUntilTimeout < -400) {
								done = true;
								SC.ui.setElementContent(nextParagraph, SC.res['JoinPanel.Launcher.NextDetectedText']);
							} else if (millisecondsUntilTimeout < 0) {
								SC.ui.dispatchCommand(nextParagraph, nextParagraph, nextParagraph, 'Load', nextPanelStringPath);
								done = true;
							} else {
								SC.ui.setElementContent(nextParagraph, SC.util.formatString(SC.res['JoinPanel.Launcher.NextTryingTextFormat'], millisecondsUntilTimeout / 1000));
							}

							if (done)
								SC.ui.removeHandler(window, 'blur', blurHandlerProc);
							else
								window.setTimeout(tickProc, 200);
						};

						SC.ui.addHandler(window, 'blur', blurHandlerProc);
						tickProc();
					}
				},
				function (container, sessionType, processType) {
					if (processType != SC.context.types.ProcessType.Guest)
						return false;

					var connectedText = SC.util.getResourceWithFallback('JoinPanel.Launcher{0}.ConnectedText', launcherKey);
					SC.ui.addElement(container, 'P', null, null, connectedText);

					if (sessionType != SC.context.types.SessionType.Meeting) {
						addHelpPanelsProc(container, containsCapabilityFunc, [
							[SC.util.Caps.Windows, helpPanels.WindowsGuestConnected],
							[SC.util.Caps.MacOSX, helpPanels.MacGuestConnected],
						]);
					}

					return true;
				}
			);
		};

		var createHandlerLauncherPanelDefinitionFunc = function (launcherKey, containsCapabilityFunc, parentSelectionHeadingText, shouldRememberSelection, joinContentBuilderProc) {
			return createBaseLauncherPanelDefinitionFunc(
				launcherKey,
				containsCapabilityFunc,
				parentSelectionHeadingText,
				shouldRememberSelection,
				0,
				function (clp) {
					var url = getLauncherUrlFunc(launcherKey, clp);
					SC.util.launchUrl(url);
				},
				joinContentBuilderProc
			);
		};

		var getLauncherUrlFunc = function (launcherKey, clientLaunchParameters) {
			return SC.context.scriptBaseUrl + SC.context.handlerPaths[launcherKey] + SC.util.getQueryString(clientLaunchParameters);;
		}

		var getRelaySchemeUrlFunc = function (clientLaunchParameters) {
			return SC.util.formatString(
				'relay://{0}:{1}/{2}/{3}/{4}/{5}/{6}/{7}',
				clientLaunchParameters.h,
				clientLaunchParameters.p,
				clientLaunchParameters.s,
				encodeURIComponent(clientLaunchParameters.k || ''),
				encodeURIComponent(clientLaunchParameters.n || ''),
				encodeURIComponent(clientLaunchParameters.r || ''),
				clientLaunchParameters.e,
				encodeURIComponent(clientLaunchParameters.i || '')
			);
		}

		var getAndroidPackageNameFunc = function () {
			if (navigator.userAgent.match(/Samsung|GT-|SCH-|SGH-|SM-/))
				return 'com.elsitech.screenconnect.androidclient.samsung';
			else
				return 'com.elsitech.screenconnect.androidclient';
		}

		var addChildPanelListProc = function (container, childPanels, childStringPaths) {
			var childPanelList = SC.ui.addElement(container, 'UL');

			for (var i = 0; i < childPanels.length; i++) {
				var listItem = SC.ui.addElement(childPanelList, 'LI');
				SC.ui.addElement(listItem, 'A', { _commandName: 'Navigate', _commandArgument: childStringPaths[i] }, null, childPanels[i].parentSelectionHeadingText);
			};
		};

		var addChildPanelDecoratedListProc = function (container, childPanels, childStringPaths) {
			for (var i = 0; i < childPanels.length; i++) {
				var panel = SC.ui.addElement(container, 'DIV', { className: 'Box' });
				SC.ui.addElement(panel, 'INPUT', { type: 'button', value: childPanels[i].parentSelectionButtonText, _commandName: 'Navigate', _commandArgument: childStringPaths[i] });
				SC.ui.addElement(panel, 'H3', null, null, childPanels[i].parentSelectionHeadingText);
				SC.ui.addElement(panel, 'P', null, null, childPanels[i].parentSelectionDescriptionText);
			}
		};

		var addHelpPanelsProc = function (container, containsCapabilityFunc, helpItems) {
			var selectedhelpPanels = SC.util.selectFromList(
				SC.util.filterList(helpItems, function (hi) {
					if (hi[0] != null) {
						if (hi[0] instanceof Array) {
							for (var i = 0; hi[0][i]; i++)
								if (!containsCapabilityFunc(hi[0][i]))
									return false;
						} else {
							if (!containsCapabilityFunc(hi[0]))
								return false;
						}
					}

					return true;

					return hi[0] == null || containsCapabilityFunc(hi[0]);
				}),
				function (hi) { return hi[1]; }
			);

			for (var i = 0; selectedhelpPanels[i]; i++) {
				var panel = SC.ui.addElement(container, 'DIV', { className: 'Box' });

				SC.ui.addElement(panel, 'IMG', { src: SC.context.scriptBaseUrl + 'Images/Launch' + selectedhelpPanels[i] + '.png' });

				if (selectedhelpPanels.length > 1)
					SC.ui.addElement(panel, 'H4', null, null, (i + 1).toString());

				var helpText = SC.res['JoinPanel.Launcher.Help' + selectedhelpPanels[i] + 'Text'];
				var helpParagraph = SC.ui.addElement(panel, 'P');
				SC.ui.setElementContent(helpParagraph, helpText, true);
			}
		};

		var addBackParagraphProc = function (container, nextPanelStringPath) {
			var backParagraph = SC.ui.addElement(container, 'P');
			SC.ui.addElement(backParagraph, 'SPAN', null, null, SC.res['JoinPanel.Launcher.BackInstalledText']);
			SC.ui.addNonBreakingSpace(backParagraph);
			SC.ui.addElement(backParagraph, 'A', { _commandName: 'Navigate', _commandArgument: nextPanelStringPath }, null, SC.res['JoinPanel.Launcher.BackJoinText']);
		};

		var createLauncherPanelDefinitionFunc = function (launcher, containsCapabilityFunc, parentSelectionHeadingTextOverride) {
			switch (launcher) {
				case launchers.ClickOnceRun:
					return createHandlerLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						true,
						function (container) {
							addHelpPanelsProc(container, containsCapabilityFunc, [
								[SC.util.Caps.Chrome, helpPanels.ChromeFileRun],
								[SC.util.Caps.Safari, helpPanels.MacSafariFileRun],
								[SC.util.Caps.Firefox, helpPanels.WindowsFirefoxExeConfirmation],
								[SC.util.Caps.Firefox, helpPanels.WindowsFirefoxExeRun],
								[SC.util.Caps.InternetExplorer, helpPanels.WindowsInternetExplorerExeRun],
								[SC.util.Caps.Windows, helpPanels.WindowsExeConfirmation],
							]);
						}
					);
				case launchers.ClickOnceBootstrap:
					return createHandlerLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						true,
						function (container) {
							addHelpPanelsProc(container, containsCapabilityFunc, [
								[SC.util.Caps.Chrome, helpPanels.ChromeFileRun],
								[SC.util.Caps.Safari, helpPanels.MacSafariFileRun],
								[SC.util.Caps.Firefox, helpPanels.WindowsFirefoxExeConfirmation],
								[SC.util.Caps.Firefox, helpPanels.WindowsFirefoxExeRun],
								[SC.util.Caps.InternetExplorer, helpPanels.WindowsInternetExplorerExeRun],
								[SC.util.Caps.Windows, helpPanels.WindowsExeConfirmation],
								[SC.util.Caps.Windows, helpPanels.WindowsClickOnceConfirmation],
							]);
						}
					);
				case launchers.WebStartDirect:
				case launchers.WebStartBootstrap:
					return createHandlerLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						true,
						function (container) {
							addHelpPanelsProc(container, containsCapabilityFunc, [
								[SC.util.Caps.Chrome, helpPanels.ChromeDangerousFileConfirmation],
								[SC.util.Caps.Chrome, helpPanels.ChromeFileRun],
								[SC.util.Caps.Safari, helpPanels.MacSafariFileRun],
								[[SC.util.Caps.MacOSX, SC.util.Caps.Firefox], helpPanels.MacFirefoxJnlpConfirmation],
								[[SC.util.Caps.Windows, SC.util.Caps.Firefox], helpPanels.WindowsFirefoxJnlpConfirmation],
								[SC.util.Caps.Windows, helpPanels.WindowsWebStartConfirmation],
								[SC.util.Caps.MacOSX, helpPanels.MacFileConfirmation],
								[SC.util.Caps.MacOSX, helpPanels.MacWebStartConfirmation],
							]);
						}
					);
				case launchers.ClickOnceDirect:
					return createBaseLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						true,
						0,
						function (clp) {
							var url = getLauncherUrlFunc(launcher, clp);
							var launched = false;

							if (SC.util.isCapable(SC.util.Caps.Chrome)) {
								try {
									// this is specific to Chrome plugin for ClickOnce to prevent page reload
									var embedElement = SC.ui.get('clickOnceEmbed') || SC.ui.addElement(document.body, 'EMBED', { id: 'clickOnceEmbed', type: 'application/x-ms-application' }, 'width: 0px; height: 0px;');
									embedElement.launchClickOnce(url);
									launched = true;
								} catch (ex) {
									// don't care, will launch normally
								}
							}

							if (!launched)
								SC.util.launchUrl(url);
						},
						function (container) {
							addHelpPanelsProc(container, containsCapabilityFunc, [
								[SC.util.Caps.Windows, helpPanels.WindowsClickOnceConfirmation],
							]);
						}
					);
				case launchers.UrlLaunch:
					return createBaseLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						true,
						3000,
						function (clp) {
							var relayUrl = getRelaySchemeUrlFunc(clp);
							SC.util.launchUrl(relayUrl);
						},
						function (container) {
							addHelpPanelsProc(container, containsCapabilityFunc, [
								[SC.util.Caps.Chrome, helpPanels.MacChromeUrlConfirmation],
								[SC.util.Caps.Firefox, helpPanels.MacFirefoxUrlConfirmation],
							]);
						}
					);
				case launchers.MacBundleSelector:
					return createPanelDefinitionFunc(
						launcher,
						parentSelectionHeadingTextOverride,
						SC.res['JoinPanel.LauncherSelector.ButtonText'],
						SC.res['JoinPanel.LauncherMacBundleSelector.Description'],
						[
							createLauncherPanelDefinitionFunc(launchers.MacBundleDownload, containsCapabilityFunc, SC.res['JoinPanel.LauncherMacBundleSelectorDownload.Heading']),
							createLauncherPanelDefinitionFunc(launchers.UrlLaunch, containsCapabilityFunc, SC.res['JoinPanel.LauncherMacBundleSelectorLaunch.Heading']),
						],
						false,
						true,
						null,
						function (container, childPanels, childStringPaths) {
							addChildPanelDecoratedListProc(container, childPanels, childStringPaths);
						}
					);
				case launchers.AndroidSelector:
					return createPanelDefinitionFunc(
						launcher,
						parentSelectionHeadingTextOverride,
						SC.res['JoinPanel.LauncherSelector.ButtonText'],
						SC.res['JoinPanel.LauncherAndroidSelector.Description'],
						[
							createLauncherPanelDefinitionFunc(launchers.PlayStore, containsCapabilityFunc, SC.res['JoinPanel.LauncherAndroidSelectorInstall.Heading']),
							createLauncherPanelDefinitionFunc(launchers.UrlLaunch, containsCapabilityFunc, SC.res['JoinPanel.LauncherAndroidSelectorLaunch.Heading']),
						],
						false,
						true,
						null,
						function (container, childPanels, childStringPaths) {
							addChildPanelDecoratedListProc(container, childPanels, childStringPaths);
						}
					);
				case launchers.AndroidIntent:
					return createBaseLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						true,
						0,
						function (clp) {
							var relayUrl = getRelaySchemeUrlFunc(clp);
							var packageName = getAndroidPackageNameFunc();
							var intentUrl = relayUrl.replace('relay:', 'intent:') + '#Intent;scheme=relay;package=' + packageName + ';end';
							SC.util.launchUrl(intentUrl);
						},
						function (container) {
						}
					);
				case launchers.MacBundleDownload:
					return createHandlerLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						false,
						function (container) {
							addHelpPanelsProc(container, containsCapabilityFunc, [
								[SC.util.Caps.Firefox, helpPanels.MacFirefoxBundleConfirmation],
								[SC.util.Caps.Firefox, helpPanels.MacFirefoxBundleRun],
								[SC.util.Caps.Firefox, helpPanels.MacBundleExtraction],
								[SC.util.Caps.Chrome, helpPanels.ChromeFileRun],
								[SC.util.Caps.Chrome, helpPanels.MacBundleExtraction],
								[SC.util.Caps.Safari, helpPanels.MacSafariFileRun],
								[SC.util.Caps.MacOSX, helpPanels.MacFileConfirmation],
							]);
						}
					);
				case launchers.PlayStore:
					return createBaseLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						false,
						0,
						function (clp) {
							var relayUrl = SC.launch.getRelaySchemeUrl(clp);
							var packageName = getAndroidPackageNameFunc();
							var url = 'market://details' + SC.util.getQueryString({ id: packageName, referrer: relayUrl });
							SC.util.launchUrl(url);
						},
						function (container, childPanels, childStringPaths, nextPanelDefinition, nextPanelStringPath) {
							addBackParagraphProc(container, nextPanelStringPath);
						}
					);
				case launchers.AppStore:
					return createBaseLauncherPanelDefinitionFunc(
						launcher,
						containsCapabilityFunc,
						parentSelectionHeadingTextOverride,
						false,
						0,
						function (clp) {
							SC.util.launchUrl('http://itunes.apple.com/us/app/screenconnect/id423995707');
						},
						function (container, childPanels, childStringPaths, nextPanelDefinition, nextPanelStringPath) {
							addBackParagraphProc(container, nextPanelStringPath);
						}
					);
			}
		}

		var processTreeItemsProc = function (systemProfiles, items, pathBuilder, score, stillScoring, capabilities) {
			SC.util.forEach(items, function (item) {
				processTreeItemProc(systemProfiles, item, pathBuilder, score, stillScoring, capabilities);
			});
		};

		var processTreeItemProc = function (systemProfiles, item, pathBuilder, score, stillScoring, capabilities) {
			var passedTest = null;

			if (item[0] instanceof Array) {
				passedTest = SC.util.isCapable(capabilities[capabilities.length - 1], item[0][0], item[0][1]);
				pathBuilder += '[' + (item[0][0] || 'X') + '-' + (item[0][1] || 'X') + ']';
			} else if (item[0] !== null) {
				passedTest = SC.util.isCapable(item[0]);
				capabilities = capabilities.slice(0).concat(item[0]);
				pathBuilder += (pathBuilder.length == 0 ? '' : ':') + SC.util.getEnumValueName(SC.util.Caps, item[0]);
			} else {
				pathBuilder += (pathBuilder.length == 0 ? '' : ':') + 'Default';
			}

			if (stillScoring && passedTest !== null) {
				if (passedTest) {
					score += 1;
				} else {
					score -= 100;
					stillScoring = false;
				}
			}

			var args = item.slice(1);

			if (item[1] instanceof Array)
				processTreeItemsProc(systemProfiles, args, pathBuilder, score, stillScoring, capabilities);
			else
				systemProfiles.push({ score: score, path: pathBuilder, launchers: args, capabilities: capabilities });
		};

		var launchers = SC.util.createEnum([
			'ClickOnceDirect',
			'ClickOnceRun',
			'ClickOnceBootstrap',
			'WebStartBootstrap',
			'WebStartDirect',
			'MacBundleDownload',
			'UrlLaunch',
			'MacBundleSelector',
			'AndroidSelector',
			'AndroidIntent',
			'PlayStore',
			'AppStore',
		]);

		var helpPanels = SC.util.createEnum([
			'ChromeDangerousFileConfirmation',
			'ChromeFileRun',
			'MacBundleExtraction',
			'MacChromeUrlConfirmation',
			'MacFileConfirmation',
			'MacFirefoxBundleConfirmation',
			'MacFirefoxBundleRun',
			'MacFirefoxJnlpConfirmation',
			'MacFirefoxUrlConfirmation',
			'MacGuestConnected',
			'MacSafariFileRun',
			'MacWebStartConfirmation',
			'WindowsClickOnceConfirmation',
			'WindowsExeConfirmation',
			'WindowsFirefoxExeConfirmation',
			'WindowsFirefoxExeRun',
			'WindowsFirefoxJnlpConfirmation',
			'WindowsGuestConnected',
			'WindowsInternetExplorerExeRun',
			'WindowsWebStartConfirmation',
		]);

		var systemProfileTree = [
			[SC.util.Caps.Windows,
				[[6, null],
					[SC.util.Caps.InternetExplorer,
						launchers.ClickOnceDirect,
						launchers.WebStartBootstrap,
						launchers.ClickOnceRun,
						launchers.WebStartDirect,
					],
					[SC.util.Caps.Chrome,
						[SC.util.Caps.ClickOnce,
							launchers.ClickOnceDirect,
							launchers.WebStartBootstrap,
							launchers.ClickOnceRun,
							launchers.WebStartDirect,
						],
						[null,
							launchers.ClickOnceRun,
							launchers.WebStartBootstrap,
							launchers.WebStartDirect,
						],
					],
					[SC.util.Caps.Firefox,
						[SC.util.Caps.ClickOnce,
							launchers.ClickOnceDirect,
							launchers.WebStartBootstrap,
							launchers.ClickOnceRun,
							launchers.WebStartDirect,
						],
						[SC.util.Caps.WebStart,
							launchers.WebStartBootstrap,
							launchers.ClickOnceRun,
							launchers.WebStartDirect,
						],
						[null,
							launchers.ClickOnceRun,
							launchers.WebStartBootstrap,
							launchers.WebStartDirect,
						],
					],
					[null,
						launchers.ClickOnceRun,
						launchers.WebStartBootstrap,
						launchers.WebStartDirect,
					],
				],
				[[5, 6],
					[SC.util.Caps.InternetExplorer,
						[SC.util.Caps.ClickOnce,
							launchers.ClickOnceDirect,
							launchers.WebStartBootstrap,
							launchers.ClickOnceBootstrap,
							launchers.WebStartDirect,
						],
						[null,
							launchers.ClickOnceBootstrap,
							launchers.WebStartBootstrap,
							launchers.WebStartDirect,
						],
					],
					[SC.util.Caps.Chrome,
						[SC.util.Caps.ClickOnce,
							launchers.ClickOnceDirect,
							launchers.WebStartBootstrap,
							launchers.ClickOnceBootstrap,
							launchers.WebStartDirect,
						],
						[null,
							launchers.ClickOnceBootstrap,
							launchers.WebStartBootstrap,
							launchers.WebStartDirect,
						],
					],
					[SC.util.Caps.Firefox,
						[SC.util.Caps.ClickOnce,
							launchers.ClickOnceDirect,
							launchers.WebStartBootstrap,
							launchers.ClickOnceBootstrap,
							launchers.WebStartDirect,
						],
						[SC.util.Caps.WebStart,
							launchers.WebStartBootstrap,
							launchers.ClickOnceBootstrap,
							launchers.WebStartDirect,
						],
						[null,
							launchers.ClickOnceBootstrap,
							launchers.WebStartBootstrap,
							launchers.WebStartDirect,
						],
					],
					[null,
						launchers.ClickOnceBootstrap,
						launchers.WebStartBootstrap,
						launchers.WebStartDirect,
					],
				]
			],
			[SC.util.Caps.MacOSX,
				[[10.9, null],
					[SC.util.Caps.Safari,
						launchers.MacBundleSelector,
						launchers.WebStartDirect,
					],
				],
				[[10.7, 10.9],
					[SC.util.Caps.Safari,
						launchers.UrlLaunch,
						launchers.MacBundleDownload,
						launchers.WebStartDirect,
					],
				],
				[[10.7, null],
					[SC.util.Caps.Chrome,
						launchers.MacBundleSelector,
						launchers.WebStartDirect,
					],
					[SC.util.Caps.Firefox,
						launchers.MacBundleSelector,
						launchers.WebStartDirect,
					],
					[null,
						launchers.MacBundleSelector,
						launchers.WebStartDirect,
					],
				],
				[null,
					launchers.WebStartDirect,
				],
			],
			[SC.util.Caps.Android,
				[SC.util.Caps.WebKit,
					[SC.util.Caps.NativeClient,
						launchers.UrlLaunch,
					],
					[[537, null],
						launchers.AndroidIntent,
					],
				],
				[SC.util.Caps.Firefox,
					launchers.AndroidSelector,
				],
				[null,
					launchers.UrlLaunch,
					launchers.PlayStore,
				],
			],
			[SC.util.Caps.iOS,
				[SC.util.Caps.NativeClient,
					launchers.UrlLaunch,
				],
				[null,
					launchers.UrlLaunch,
					launchers.AppStore,
				],
			],
			[null,
				launchers.WebStartDirect,
				launchers.UrlLaunch,
			],
		];

		(function () {
			var systemProfiles = [];
			processTreeItemsProc(systemProfiles, systemProfileTree, '', 0, true, []);

			systemProfiles.sort(function (x, y) {
				var scoreResult = y.score - x.score;
				return (scoreResult != 0 ? scoreResult : x.path.localeCompare(y.path));
			});

			var systemProfilePanelDefinitions = SC.util.selectFromList(systemProfiles, function (sc) {
				return createPanelDefinitionFunc(
					sc.path,
					null,
					null,
					null,
					SC.util.selectFromList(sc.launchers, function (launcher) {
						var containsCapabilityFunc = function (capability) { return SC.util.indexOf(sc.capabilities, capability) != -1; };
						return createLauncherPanelDefinitionFunc(launcher, containsCapabilityFunc);
					}),
					true,
					false,
					null,
					function (container, childPanels, childStringPaths) {
						SC.ui.addElement(container, 'P', null, null, SC.res['JoinPanel.Launcher.LauncherMessage']);
						addChildPanelDecoratedListProc(container, childPanels, childStringPaths);
					}
				);
			});

			window._joinPanelDefinitionTreeRoot = createPanelDefinitionFunc(
				'SystemProfiles',
				null,
				null,
				null,
				systemProfilePanelDefinitions,
				true,
				false,
				null,
				function (container, childPanels, childStringPaths) {
					SC.ui.addElement(container, 'P', null, null, SC.res['JoinPanel.Launcher.SystemProfileMessage']);
					addChildPanelListProc(container, childPanels, childStringPaths);
				}
			);
		})();
	}

	return window._joinPanelDefinitionTreeRoot;
}

// for legacy compatibility
SC.launch.getSortedLaunchers = function () {
	return [{
		launch: function (clientLaunchParameters) {
			var joinPanelDefinitionTreeRoot = SC.launch.getJoinPanelDefinitionTreeRoot();
			var initialPanelDefinitionPath = SC.launch.getInitialJoinPanelDefinitionPath(joinPanelDefinitionTreeRoot);
			var panelDefinition = initialPanelDefinitionPath[initialPanelDefinitionPath.length - 1];

			if (panelDefinition.joinProc != null)
				panelDefinition.joinProc(clientLaunchParameters);
		}
	}];
}

SC.launch.getPanelDefinitionPath = function (stringPath, panelDefinitionTreeRoot) {
	var panelDefinitionPath = [panelDefinitionTreeRoot];

	if (stringPath != null) {
		var stringPathParts = stringPath.split('/');

		for (var i = 0; i < stringPathParts.length; i++) {
			var pathPart = SC.util.findListElement(panelDefinitionPath[panelDefinitionPath.length - 1].childPanels, function (cp) { return cp.key == stringPathParts[i]; });

			if (pathPart == null)
				return null;

			panelDefinitionPath.push(pathPart);
		}
	}

	return panelDefinitionPath;
};

SC.launch.getInitialJoinPanelDefinitionPath = function (joinPanelDefinitionTreeRoot) {
	var panelDefinitionPath = null;
	var settings = SC.util.loadSettings();

	if (settings.joinPath)
		panelDefinitionPath = SC.launch.getPanelDefinitionPath(settings.joinPath, joinPanelDefinitionTreeRoot);

	if (panelDefinitionPath == null) {
		panelDefinitionPath = [joinPanelDefinitionTreeRoot];

		while (panelDefinitionPath[panelDefinitionPath.length - 1].shouldAutoNavigateToFirstChild)
			panelDefinitionPath.push(panelDefinitionPath[panelDefinitionPath.length - 1].childPanels[0]);
	}

	return panelDefinitionPath;
}

SC.launch.startJoinSession = function (
	sessionToken,
	getClientLaunchParametersFunc,
	getJoiningParticipantNameFunc,
	getJoinedParticipantCountFunc,
	shouldShowPrompt,
	promptText,
	promptFieldMap,
	promptButtonText
) {
	var getStringPathFunc = function (panelDefinitionPath) {
		if (panelDefinitionPath.length == 1)
			return null;

		return SC.util.selectFromList(panelDefinitionPath.slice(1), function (pd) { return pd.key; }).join('/');
	};

	var renderPromptProc = function () {
		SC.ui.clearElement(contentPanel);
		SC.ui.setVisible(buttonPanel, true);

		var promptParagraph = SC.ui.addElement(contentPanel, 'P');
		SC.ui.setElementContent(promptParagraph, promptText, true);

		var definitionList = SC.ui.addElement(contentPanel, 'DL');
		var button = SC.ui.addElement(buttonPanel, 'INPUT', { type: 'button', _commandName: 'Enter', value: promptButtonText });
		var focused = false;

		for (var promptFieldKey in promptFieldMap) {
			var promptField = promptFieldMap[promptFieldKey];

			if (promptField.visible) {
				var inputElement = promptField.type == 'L' ? SC.ui.createElement('SPAN', null, null, promptField.value) :
				SC.ui.createTextBox(promptField.type == 'B', promptField.type == 'P', null, 'Enter');

				if (typeof inputElement.value !== "undefined") {
					inputElement.value = promptField.value;
					inputElement._promptField = promptField;
				}

				SC.ui.addElement(definitionList, 'DT', null, null, promptField.labelText);
				SC.ui.addElement(definitionList, 'DD', null, null, inputElement);

				if (!focused) {
					inputElement.focus()
					focused = true;
				}
			}
		};

		if (!focused)
			button.focus();
	};

	var renderJoinProc = function (panelDefinitionPath) {
		var panelDefinition = panelDefinitionPath[panelDefinitionPath.length - 1];

		contentPanel.scrollTop = 0;
		SC.ui.clearElement(contentPanel);

		SC.ui.setVisible(buttonPanel, false);

		if (panelDefinition.joinContentBuilderProc != null) {
			var childPanelStringPaths = panelDefinition.childPanels == null ? null : SC.util.selectFromList(panelDefinition.childPanels, function (cpd) { return getStringPathFunc(panelDefinitionPath.concat(cpd)); });
			var parentPanelDefinition = panelDefinitionPath[panelDefinitionPath.length - 2] || null;
			var parentPanelDefinitionPath = panelDefinitionPath.slice(0, panelDefinitionPath.length - 1);
			var selectedChildIndex = (parentPanelDefinition == null ? -1 : SC.util.indexOf(parentPanelDefinition.childPanels, panelDefinition));
			var nextPanelDefinition = (parentPanelDefinition == null || parentPanelDefinition.length == 1 ? null : parentPanelDefinition.childPanels[(selectedChildIndex + 1) % parentPanelDefinition.childPanels.length]);
			var nextPanelStringPath = (nextPanelDefinition == null ? null : getStringPathFunc(parentPanelDefinitionPath.concat(nextPanelDefinition)));
			panelDefinition.joinContentBuilderProc(contentPanel, panelDefinition.childPanels, childPanelStringPaths, nextPanelDefinition, nextPanelStringPath);
		}

		var lineageParagraph = SC.ui.addElement(contentPanel, 'P', { className: 'Help' });
		var lineagePanelDefinitionPath = [];

		SC.util.forEach(panelDefinitionPath, function (pd) {
			if (lineagePanelDefinitionPath.length != 0)
				SC.ui.addElement(lineageParagraph, 'SPAN', null, null, ' / ');

			lineagePanelDefinitionPath.push(pd);

			var stringPath = getStringPathFunc(lineagePanelDefinitionPath);

			if (lineagePanelDefinitionPath.length == panelDefinitionPath.length)
				SC.ui.addElement(lineageParagraph, 'SPAN', null, null, pd.key);
			else
				SC.ui.addElement(lineageParagraph, 'A', { _commandName: 'Navigate', _commandArgument: stringPath }, null, pd.key);
		});

		if (panelDefinition.joinProc != null) {
			joiningPanelDefinition = panelDefinition;
			joiningParticipantName = getJoiningParticipantNameFunc(joiningClientLaunchParameters),
			joiningParticipantCount = getJoinedParticipantCountFunc(joiningClientLaunchParameters, joiningParticipantName);

			joiningPanelDefinition.joinProc(joiningClientLaunchParameters);

			var joinData = SC.util.formatString('({0}) {1}', joiningPanelDefinition.key, window.navigator.userAgent);
			SC.service.logInitiatedJoin(joiningClientLaunchParameters.s, joinData);
		}
	};

	var endPromptProc = function () {
		getClientLaunchParametersFunc(
			sessionToken,
			promptFieldMap,
			function (clientLaunchParameters) {
				joiningClientLaunchParameters = clientLaunchParameters;

				var initialPanelDefinitionPath = SC.launch.getInitialJoinPanelDefinitionPath(joinPanelDefinitionTreeRoot);
				renderJoinProc(initialPanelDefinitionPath);
			},
			function (error) {
				SC.ui.setModalDialogButtonPanelError(buttonPanel, error);
			}
		);
	};

	var renderConnectedFunc = function () {
		SC.ui.setVisible(buttonPanel, false);
		SC.ui.clearElement(contentPanel);

		var processType = SC.context.types.ProcessType[joiningClientLaunchParameters.y];
		var sessionType = SC.context.types.SessionType[joiningClientLaunchParameters.e];
		return joiningPanelDefinition.connectedContentBuilderFunc(contentPanel, sessionType, processType);
	};

	var commandProc = function (command) {
		if (command.commandName == 'Navigate' || command.commandName == 'Load') {
			var panelDefinitionPath = SC.launch.getPanelDefinitionPath(command.commandArgument, joinPanelDefinitionTreeRoot);

			if (command.commandName == 'Navigate' && panelDefinitionPath != null && panelDefinitionPath[panelDefinitionPath.length - 1].shouldRememberSelection) {
				var settings = SC.util.loadSettings();
				settings.joinPath = command.commandArgument;
				SC.util.saveSettings(settings);
			}

			renderJoinProc(panelDefinitionPath);
		} else if (command.commandName == 'Enter') {
			SC.ui.findDescendentNode(contentPanel, function (n) {
				if (typeof n._promptField !== "undefined")
					n._promptField.value = n.value;
			});

			endPromptProc();
		}
	};

	var refreshProc = function () {
		if (joiningParticipantName != null) {
			var joinedParticipantCount = getJoinedParticipantCountFunc(joiningClientLaunchParameters, joiningParticipantName);

			if (joinedParticipantCount > joiningParticipantCount) {
				joiningParticipantName = null;

				if (!renderConnectedFunc())
					SC.ui.hideModalDialog();
			}
		}
	};

	var joiningClientLaunchParameters = null;
	var joiningParticipantName = null
	var joiningParticipantCount = null;
	var joiningPanelDefinition = null;
	var joinPanelDefinitionTreeRoot = SC.launch.getJoinPanelDefinitionTreeRoot();
	var titlePanel = SC.ui.createModalDialogTitlePanel(SC.res['JoinPanel.Title']);
	var contentPanel = SC.ui.createModalDialogContentPanel();
	var buttonPanel = SC.ui.createModalDialogButtonPanel();

	(function () {
		SC.ui.showModalDialogRaw('JoinSession', [titlePanel, contentPanel, buttonPanel], commandProc, refreshProc);

		if (shouldShowPrompt)
			renderPromptProc();
		else
			endPromptProc();

		refreshProc();
	})();
};
