﻿window.SC = window.SC || {};
SC.util = SC.util || {};

SC.util.jsonSerialize = function (object) {
	if (typeof Sys !== "undefined") {
		return Sys.Serialization.JavaScriptSerializer.serialize(object);
	} else if (typeof JSON !== "undefined") {
		if (window.Prototype)
			delete Array.prototype.toJSON;

		return JSON.stringify(object);
	} else if (typeof object.toJSON !== "undefined") {
		return object.toJSON();
	}
}

SC.util.jsonDeserialize = function (jsonString) {
	if (typeof Sys !== "undefined")
		return Sys.Serialization.JavaScriptSerializer.deserialize(jsonString);
	else if (typeof JSON !== "undefined")
		return JSON.parse(jsonString);
	else if (typeof jsonString.evalJSON !== "undefined")
		return jsonString.evalJSON(true);
}

SC.util.formatString = function (format, args) {
	var actualArguments = arguments;
	var actualArgumentsOffset = 1;

	if (args instanceof Array) {
		actualArguments = args;
		actualArgumentsOffset = 0;
	}

	var replaceFunc = function (a, b, c, d) {
		var argIndex = parseInt(b);
		var argFormat = d;
		var argValue = actualArguments[argIndex + actualArgumentsOffset];

		if (argValue == undefined || argValue == null)
			return "";
		else if (d != undefined && argValue.toFormattedString)
			return argValue.toFormattedString(d);
		else if (d != undefined && argValue.format)
			return argValue.format(d);
		else
			return argValue.toString();
	};

	return format.replace(/\{([0-9]+)(:([^\}]+))?\}/g, replaceFunc);
}

SC.util.generatePhoneticText = function (string) {
	var phoneticCodes = SC.util.selectFromList(string, function (codeChar) { return SC.res["PhoneticAlphabet." + codeChar.toUpperCase()] || codeChar });
	return phoneticCodes.join("-");
}

SC.util.getCookieValue = function (name) {
	var cookieStrings = document.cookie.split(";");

	for (var i = 0; cookieStrings[i]; i++) {
		var cookieParts = cookieStrings[i].split("=");

		if (decodeURIComponent(cookieParts[0]).trim() == name)
			return decodeURIComponent(cookieParts[1]);
	}

	return null;
}

SC.util.setCookieValue = function (name, value, days) {
	var date = SC.util.addDateDays(new Date(), days);
	var encodedValue = encodeURIComponent(value);
	document.cookie = SC.util.formatString("{0}={1}; expires={2:F}", name, encodedValue, date);
}

SC.util.loadSettings = function () {
	var stringValue = SC.util.getCookieValue("settings");
	
	try {
		if (!SC.util.isNullOrEmpty(stringValue))
			return SC.util.jsonDeserialize(stringValue);
	} catch (ex) {
		// don't care
	}

	return {};
}

SC.util.saveSettings = function (settings) {
	var stringValue = SC.util.jsonSerialize(settings);
	SC.util.setCookieValue("settings", stringValue, 3650);
}

SC.util.parseQueryString = function (queryString) {
	var map = {};

	if (queryString.length != 0) {
		if (queryString.charAt(0) == "?")
			queryString = queryString.slice(1);

		queryString = queryString.replace(/\+/g, ' ');

		var parts = queryString.split("&");

		for (var i = 0; parts[i]; i++) {
			var subParts = parts[i].split("=");
			var name = decodeURIComponent(subParts[0]);
			var value = decodeURIComponent(subParts[1]);
			map[name] = value;
		}
	}

	return map;
}

SC.util.getQueryString = function (map) {
	var queryString = "";
	var first = true;

	for (var key in map) {
		var value = map[key];
		var valueArray;

		if (value == undefined || value == null) {
			valueArray = [];
		} else if (value instanceof Array) {
			valueArray = value;
		} else {
			// watch out, in one JS version Array constructor with integer
			// will create that many elements
			valueArray = new Array();
			valueArray.push(value);
		}

		for (var i = 0; i < valueArray.length; i++) {
			queryString += (first ? "?" : "&") + encodeURIComponent(key) + "=" + encodeURIComponent(valueArray[i]);
			first = false;
		}
	}

	return queryString;
}

SC.util.launchUrl = function (url) {
	if (SC.util.isCapable(SC.util.Caps.MacOSX) && SC.util.isCapable(SC.util.Caps.Chrome) && window.location.protocol == "https:")
		window.location.assign(url); // required to not be flagged when opening different scheme from https
	else if (SC.util.isMobile())
		window.location.href = url;
	else
		SC.ui.addElement(document.body, "IFRAME", { src: url }, "display: none;");
}

SC.util.areAllEqual = function (array, selector) {
	var firstValue = (selector ? selector(array[0]) : array[0]);

	for (var i = 1; array[i] != undefined; i++)
		if ((selector ? selector(array[i]) : array[i]) != firstValue)
			return false;

	return true;
}

SC.util.contains = function (array, predicate) {
	return SC.util.findListIndex(array, predicate) != -1;
}

SC.util.findListIndex = function (array, predicate) {
	if (array != null)
		for (var i = 0; array[i] != undefined; i++)
			if (predicate(array[i], i))
				return i;

	return -1;
}

SC.util.findListElement = function (array, predicate) {
	var index = SC.util.findListIndex(array, predicate);
	return (index == -1 ? null : array[index]);
}

SC.util.filterList = function (array, predicate) {
	var newArray = [];

	if (array != null)
		for (var i = 0; array[i] != undefined; i++)
			if (predicate(array[i], i))
				newArray.push(array[i]);

	return newArray;
}

SC.util.selectFromList = function (array, selector) {
	var newArray = [];

	if (array != null)
		for (var i = 0; array[i] != undefined; i++)
			newArray[i] = selector(array[i], i);

	return newArray;
}

SC.util.countFromList = function (array, predicate) {
	var count = 0;

	if (array != null)
		for (var i = 0; array[i] != undefined; i++)
			if (predicate(array[i], i))
				count++;

	return count;
}

SC.util.createArray = function (size, populator) {
	var array = [];

	for (var i = 0; i < size; i++)
		array[i] = populator(i);

	return array;
}

SC.util.createEnum = function (names) {
	var enumObject = {};
	SC.util.forEach(names, function (n) { enumObject[n] = n; });
	return enumObject;
};

SC.util.indexOf = function (array, item) {
	for (var i = 0; array[i] !== undefined; i++)
		if (array[i] === item)
			return i;

	return -1;
}

SC.util.forEach = function (array, func) {
	if (array != null)
		for (var i = 0; array[i] !== undefined; i++)
			func(array[i], i);
}

SC.util.coalesce = function (x, y) {
	return (x === null || x === undefined ? y : x);
}

SC.util.isVersion = function (minVersionInclusive, maxVersionExclusive, actualVersion) {
	if (actualVersion == 0)
		return false;

	if (minVersionInclusive !== undefined && minVersionInclusive !== null && actualVersion < minVersionInclusive)
		return false;

	if (maxVersionExclusive !== undefined && maxVersionExclusive !== null && actualVersion >= maxVersionExclusive)
		return false;

	return true;
}

SC.util.Caps = {
	Windows: function () { return SC.util.getUserAgentVersion(/Windows (?:NT )?([0-9.]+)/); },
	MacOSX: function () { return SC.util.getUserAgentVersion(/Mac OS X (([0-9]+)(_|\.)([0-9]+))(_|;|\))/); },
	Android: function () { return SC.util.getUserAgentVersion(/android/i); },
	iOS: function () { return SC.util.getUserAgentVersion(/iphone|ipad|ipod|ios/i); },
	InternetExplorer: function () { return SC.util.getUserAgentVersion(/MSIE ([0-9.]+)/) || SC.util.getUserAgentVersion(/Trident.*; rv:([0-9.]+\.[0-9]+)\)/); },
	Chrome: function () { return SC.util.getUserAgentVersion(/Chrome\/([0-9.]+\.[0-9]+)/); },
	Firefox: function () { return SC.util.getUserAgentVersion(/Firefox\/([0-9.]+\.[0-9]+)/); },
	Safari: function () { return SC.util.getUserAgentVersion(/Safari\/([0-9.]+\.[0-9]+)/); },
	WebKit: function () { return SC.util.getUserAgentVersion(/WebKit\/([0-9.]+)/); },
	NativeClient: function () { return SC.util.getUserAgentVersion(/ScreenConnect\/([0-9]+\.[0-9]+)\./); },
	ClickOnce: function () {
		return (SC.context.clickOnceHeaderValue != null ||
			navigator.userAgent.match(/\.NET/) ||
			SC.util.isCapable(SC.util.Caps.InternetExplorer, 9) ||
			(SC.util.isCapable(SC.util.Caps.Chrome) && window.navigator.mimeTypes["application/x-ms-application"] != null)) ? 1 : 0;
	},
	WebStart: function () {
		// IE has new ActiveXObject("JavaWebStart.isInstalled"), but it prompts to run plugin, which is bad
		if (window.navigator.mimeTypes["application/x-java-jnlp-file"] != null || navigator.mimeTypes["application/x-java-applet;version=1.5"] != null)
			return 1;

		return 0;
	},
};

SC.util.isCapable = function (capability, minVersionInclusive, maxVersionExclusive) {
	window._capabilities = window._capabilities || {};

	if (window._capabilities[capability] === undefined)
		window._capabilities[capability] = capability();

	return SC.util.isVersion(minVersionInclusive, maxVersionExclusive, window._capabilities[capability])
}

SC.util.getUserAgentVersion = function (pattern) {
	var matches = navigator.userAgent.match(pattern);
	return (matches === null ? 0 : (matches[1] === undefined ? 1 : parseFloat(matches[1].replace("_", "."))));
}

SC.util.isMobile = function () {
	return SC.util.isCapable(SC.util.Caps.iOS) || SC.util.isCapable(SC.util.Caps.Android);
}

SC.util.doesBrowserNeedSyncServiceContextForLaunch = function () {
	return SC.util.isCapable(SC.util.Caps.InternetExplorer);
}

SC.util.canSilentlyFailBadUrl = function () {
	// they do some kind of prompting
	if (SC.util.isCapable(SC.util.Caps.Android) && SC.util.isCapable(SC.util.Caps.Firefox))
		return false;

	// they broke it and now goes to error page with ERR_UNKNOWN_URL_SCHEME
	if (SC.util.isCapable(SC.util.Caps.Android) && SC.util.isCapable(SC.util.Caps.WebKit, 537))
		return false;

	// safari on mavericks now displays "There is no application set to open the URL"
	if (SC.util.isCapable(SC.util.Caps.MacOSX, 10.9) && SC.util.isCapable(SC.util.Caps.Safari))
		return false;

	// chrome always displays "External Protocol Request" prompt regardless of whether the app is installed
	if (SC.util.isCapable(SC.util.Caps.MacOSX) && SC.util.isCapable(SC.util.Caps.Chrome))
		return false;

	return true;
}

SC.util.combineObjects = function () {
	var newObject = {};

	SC.util.findListElement(arguments, function (i) {
		for (var j in i)
			newObject[j] = i[j]
	});

	return newObject;
}

SC.util.openClientEmail = function (to, subject, body) {
	var url = "mailto:" + (to == null ? "" : to) + SC.util.getQueryString({ subject: subject, body: body });
	SC.util.launchUrl(url);
}

SC.util.formatDomainMember = function (domain, member) {
	var domainEmpty = SC.util.isNullOrEmpty(domain);
	var memberEmpty = SC.util.isNullOrEmpty(member);

	if (domainEmpty && memberEmpty)
		return "";
	else if (domainEmpty)
		return member;
	else if (memberEmpty)
		return domain + "\\";
	else
		return domain + "\\" + member;
}

SC.util.formatDuration = function (originalSeconds, additionalSeconds) {
	var string = "";
	var durationSeconds = originalSeconds;

	if (additionalSeconds)
		durationSeconds += additionalSeconds;

	if (originalSeconds >= 0) {
		var days = Math.floor(durationSeconds / 86400); durationSeconds %= 86400;
		var hours = Math.floor(durationSeconds / 3600); durationSeconds %= 3600;
		var minutes = Math.floor(durationSeconds / 60); durationSeconds %= 60;

		if (days != 0)
			string += days + "d ";

		if (hours != 0)
			string += hours + "h ";

		string += minutes + "m";
	}

	return string;
}

SC.util.formatDateTime = function (dateTime) {
	return SC.util.padToTwoDigits(dateTime.getMonth() + 1) +
		"-" +
		SC.util.padToTwoDigits(dateTime.getDate()) +
		" " +
		SC.util.padToTwoDigits(dateTime.getHours()) +
		":" +
		SC.util.padToTwoDigits(dateTime.getMinutes()) +
		":" +
		SC.util.padToTwoDigits(dateTime.getSeconds());
}

SC.util.getMillisecondCount = function () {
	return new Date().getTime();
}

SC.util.addDateMilliseconds = function (date, milliseconds) {
	var originalTime = date.getTime();
	return new Date(originalTime + milliseconds);
}

SC.util.addDateSeconds = function (date, seconds) {
	return SC.util.addDateMilliseconds(date, seconds * 1000);
}

SC.util.addDateDays = function (date, days) {
	return SC.util.addDateSeconds(date, days * 86400000);
}

SC.util.padToTwoDigits = function (number) {
	return (number < 10) ? "0" + number : number.toString();
}

SC.util.getHashParameters = function () {
	var hashString = SC.util.getWindowHashString();

	if (!hashString || hashString.length < 1)
		return [];

	var hashParts = hashString.substring(1).split("/");
	return SC.util.selectFromList(hashParts, function (hashPart) { return decodeURIComponent(hashPart); });
}

SC.util.getHashStringFromParameters = function (parameters) {
	var lastParameterIndex = -1;
	var hashString = "";

	for (var i = 0; i < parameters.length; i++)
		if (!SC.util.isNullOrEmpty(parameters[i]))
			lastParameterIndex = i;

	for (var i = 0; i <= lastParameterIndex; i++) {
		hashString += (i == 0 ? "#" : "/");

		if (!SC.util.isNullOrEmpty(parameters[i]))
			hashString += encodeURIComponent(parameters[i]);
	}

	return hashString;
}

SC.util.getWindowHashString = function () {
	// bug in firefox requires using different impl than window.location.hash
	// https://bugzilla.mozilla.org/show_bug.cgi?id=582361
	var hashIndex = window.location.href.indexOf("#");
	return (hashIndex == -1 ? "" : window.location.href.substring(hashIndex));
}

SC.util.getHashParameter = function (index) {
	var hashParameters = SC.util.getHashParameters();
	return hashParameters[index];
}

SC.util.setHashParameter = function (index, value) {
	var hashParameters = SC.util.getHashParameters();

	for (var i = 0; i < index; i++)
		if (hashParameters[i] == undefined)
			hashParameters.push("");

	hashParameters[i] = value;

	var newHashString = SC.util.getHashStringFromParameters(hashParameters)
	var existingHashString = SC.util.getWindowHashString();
	var isDifferent = newHashString != existingHashString;

	if (isDifferent)
		window.location.hash = newHashString;

	return isDifferent;
}

SC.util.base64Encode = function (input) {
	var keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
	var output = "";
	var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
	var i = 0;

	while (i < input.length) {
		chr1 = input.charCodeAt(i++);
		chr2 = input.charCodeAt(i++);
		chr3 = input.charCodeAt(i++);

		enc1 = chr1 >> 2;
		enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
		enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
		enc4 = chr3 & 63;

		if (isNaN(chr2)) {
			enc3 = enc4 = 64;
		} else if (isNaN(chr3)) {
			enc4 = 64;
		}

		output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2) + keyStr.charAt(enc3) + keyStr.charAt(enc4);
	}

	return output;
}

SC.util.isNullOrEmpty = function (string) {
	return (typeof string != "string" || string == "");
}

SC.util.getEnumValueName = function (enumType, value) {
	for (var enumName in enumType)
		if (enumType[enumName] == value)
			return enumName;

	return null;
}

SC.util.getRandomStringFromMask = function (mask) {
	var string = "";

	for (var i = 0; mask[i]; i++) {
		var maskChar = mask.charAt(i);

		if (maskChar == "#")
			string += SC.util.getRandomChar(48, 58);
		else if (maskChar == "A")
			string += SC.util.getRandomChar(65, 91);
		else
			string += maskChar;
	}

	return string;
}

SC.util.getRandomChar = function (minCharCode, maxCharCode) {
	var charCode = minCharCode + Math.floor(Math.random() * (maxCharCode - minCharCode));
	return String.fromCharCode(charCode);
}

SC.util.areObjectsEqual = function (objectX, objectY) {
	for (var field in objectX) {
		if (objectX[field] != null && objectY[field] != null && typeof (objectX[field]) == "object") {
			if (!SC.util.areObjectsEqual(objectX[field], objectY[field]))
				return false;
		} else {
			if (objectX[field] !== objectY[field])
				return false;
		}
	}

	return true;
}

SC.util.stringToBoolean = function (string) {
	return string === "true";
}

SC.util.getResourceWithFallback = function (resourceNameFormat, varArgs) {
	var iterationCount = 1 << (arguments.length - 1);

	for (var i = 0; i < iterationCount; i++) {
		var iterationArguments = SC.util.selectFromList(arguments, function (arg, argIndex) { return (argIndex & i) == 0 ? arg : ""; });
		var resourceName = SC.util.formatString.apply(this, iterationArguments);
		var value = SC.res[resourceName];

		if (value != undefined && value != null)
			return value;
	}

	return null;
}

SC.util.getSessionTypeResource = function (resourceNameFormat, sessionType, varArgs) {
	var argumentsCopy = Array.prototype.slice.call(arguments);
	argumentsCopy[1] = SC.util.getEnumValueName(SC.context.types.SessionType, sessionType);
	return SC.util.getResourceWithFallback.apply(this, argumentsCopy);
}

SC.util.getSessionTypeBooleanResource = function (resourceNameFormat, sessionType, varArgs) {
	var stringValue = SC.util.getSessionTypeResource.apply(this, arguments);
	return SC.util.stringToBoolean(stringValue);
}

SC.util.getBooleanResource = function (resourceName) {
	return SC.util.stringToBoolean(SC.res[resourceName]);
}

SC.util.getClientLaunchParameters = function (commonClientLaunchParameters, sessionClientLaunchParameters, sessionType, processType, participantName) {
	return SC.util.combineObjects(
		commonClientLaunchParameters,
		sessionClientLaunchParameters,
		{
			e: SC.util.getEnumValueName(SC.context.types.SessionType, sessionType),
			y: SC.util.getEnumValueName(SC.context.types.ProcessType, processType),
			r: participantName
		}
	);
}

SC.util.replaceAll = function (string, find, replace, ignoreCase) {
	return string.replace(new RegExp(find.replace(/([\/\,\!\\\^\$\{\}\[\]\(\)\.\*\+\?\|\<\>\-\&])/g, "\\$&"), (ignoreCase ? "gi" : "g")), (typeof (replace) == "string") ? replace.replace(/\$/g, "$$$$") : replace);
};

SC.util.encodeHtml = function (htmlString) {
	return SC.util.replaceAll(SC.util.replaceAll(SC.util.replaceAll(SC.util.replaceAll(SC.util.replaceAll(htmlString, "&", "&amp;"), ">", "&gt;"), "<", "&lt;"), "\"", "&quot;"), "'", "&#39;");
}

SC.util.forEachVisibleCustomProperty = function (sessionType, proc) {
	for (var i = 0; i < SC.context.customPropertyCount; i++)
		if (SC.util.getSessionTypeBooleanResource("SessionProperty.Custom{1}.{0}Visible", sessionType, i + 1))
			proc(i, "Custom" + (i + 1));
};

SC.util.isDefined = function (reference) {
	// === "function" does not work for built-in functions and IE 8
	return typeof reference !== "undefined";
}

SC.util.getCacheEntry = function (key, version) {
	window._cache = window._cache || {};

	var cacheEntry = window._cache[key];

	if (!cacheEntry || cacheEntry.version != version)
		return null;

	cacheEntry.lastUsedTime = SC.util.getMillisecondCount();
	return cacheEntry;
}

SC.util.setCacheItem = function (key, version, item) {
	window._cache = window._cache || {};

	var cacheEntry = window._cache[key];

	if (!cacheEntry) {
		cacheEntry = {};
		window._cache[key] = cacheEntry;
	}

	cacheEntry.version = version;
	cacheEntry.firstUsedTime =
	cacheEntry.lastUsedTime = SC.util.getMillisecondCount();
	cacheEntry.item = item;

	// scavenge
	if (!window._cacheIntervalID) {
		window._cacheIntervalID = window.setTimeout(function () {
			var hasEntries = false;
			var now = SC.util.getMillisecondCount();

			for (var cacheKey in window._cache) {
				if (now - window._cache[cacheKey].lastUsedTime > 120000)
					window._cache[cacheKey] = undefined;
				else
					hasEntries = true;
			}

			if (!hasEntries) {
				window.clearInterval(window._cacheIntervalID);
				window._cacheIntervalID = undefined;
			}
		}, 120000);
	}

	return cacheEntry;
}

SC.util.tryGet = function (getter) {
	try {
		return getter();
	} catch (e) {
		return null;
	}
}

SC.util.performWithServiceContext = function (syncOrAsync, proc) {
	try {
		window._inSyncServiceContext = syncOrAsync;
		proc();
	} finally {
		window._inSyncServiceContext = false;
	}
}

SC.util.invokeService = function (methodName, params, onSuccess, onFailure, userContext, userName, password) {
	var url = SC.context.scriptBaseUrl + "Service.ashx/" + methodName;
	var paramsString = SC.util.jsonSerialize(params);

	var xhr;

	var completeProc = function (isKnownError) {
		var result = SC.util.tryGet(function () { return SC.util.jsonDeserialize(xhr.responseText); });

		if (isKnownError || (xhr.status !== undefined && xhr.status !== 200)) {
			result = result || {};
			result.errorType = result.errorType || (SC.util.indexOf([0, 404, 408, 504, 598, 599, 12002, 12019], xhr.status) == -1 ? "UnknownException" : "TimeoutException");
			result.message = result.message || (SC.util.isNullOrEmpty(xhr.statusText) ? "Unknown error" : xhr.statusText);
			result.statusCode = result.statusCode || xhr.status;

			var handled = false;

			if (SC.context.prehandleServiceFailureProc)
				handled = SC.context.prehandleServiceFailureProc(result, userContext) != false;

			if (!handled && onFailure)
				handled = onFailure(result, userContext) != false;

			if (!handled && SC.context.unhandledServiceFailureProc)
				SC.context.unhandledServiceFailureProc(result, userContext);
		} else {
			if (onSuccess)
				onSuccess(result, userContext);
		}
	}

	var authorizationString = (userName && password ? "Basic " + SC.util.base64Encode(userName + ":" + password) : null);

	var sendProc = function () {
		var fullUrl = url;

		if (typeof xhr.onload !== "undefined")
			xhr.onload = function () { completeProc(false); }

		if (typeof xhr.onerror !== "undefined")
			xhr.onerror = function () { completeProc(true); }

		if (typeof xhr.onprogress !== "undefined")
			xhr.onprogress = function () { }

		if (typeof xhr.setRequestHeader === "undefined") {
			var urlParameters = {};
			urlParameters["__UnauthorizedStatusCode"] = 403;

			if (authorizationString != null)
				urlParameters["__Authorization"] = authorizationString;

			fullUrl += SC.util.getQueryString(urlParameters);
		}

		xhr.open("POST", fullUrl, !window._inSyncServiceContext);

		if (typeof xhr.onload === "undefined")
			xhr.onreadystatechange = function () { if (xhr.readyState == 4) completeProc(false); }

		if (typeof xhr.withCredentials !== "undefined")
			if (!window._inSyncServiceContext)
				xhr.withCredentials = true;

		if (typeof xhr.setRequestHeader !== "undefined") {
			xhr.setRequestHeader("X-Unauthorized-Status-Code", 403);

			if (authorizationString != null)
				xhr.setRequestHeader("Authorization", authorizationString);
		}

		xhr.send(paramsString);
	}

	try {
		xhr = new XMLHttpRequest();
		sendProc(xhr);
	} catch (e) {
		if (typeof XDomainRequest === "undefined")
			throw e;

		xhr = new XDomainRequest();
		sendProc(xhr);
	}

	return xhr;
}
