﻿<%@ Control Language="C#" ClassName="Elsinore.ScreenConnect.SecurityPanel" %>
<%@ Implements Interface="Elsinore.ScreenConnect.IDataControl" %>
<script runat="server">

	const string EmptyPassword = "CE5B9879";

	protected override void LoadViewState(object savedState)
	{
		base.LoadViewState(savedState.As<Pair>().First);

		this.userView.DataSource = savedState.As<Pair>().Second.As<Triplet>().First;
		this.roleView.DataSource = savedState.As<Pair>().Second.As<Triplet>().Second;
		this.userSourceConfigurationView.DataSource = savedState.As<Pair>().Second.As<Triplet>().Third;

		this.SetUserViewEmptyDataText();

		this.userView.DataBind();
		this.roleView.DataBind();
		this.userSourceConfigurationView.DataBind();
	}

	protected override object SaveViewState()
	{
		return new Pair(
			base.SaveViewState(),
			new Triplet(
				this.userView.DataSource,
				this.roleView.DataSource,
				this.userSourceConfigurationView.DataSource
			)
		);
	}

	protected override void OnPreRender(EventArgs e)
	{
		base.OnPreRender(e);

		this.userSourceBox.Enabled =
		this.newRoleButton.Enabled =
		this.newUserButton.Enabled = (this.roleView.EditIndex == -1 && this.userView.EditIndex == -1);

		this.userValidator.Visible =
		this.newUserButton.Visible = (this.userView.DataSource != null);

		this.userView.Columns
			.OfType<BoundField>()
			.ToArray()
			.SafeDo(fs =>
			{
				fs.First(bf => bf.DataField == "PasswordQuestion").Visible = Extensions.TryParseBool(Resources.Default.SecurityPanel_PasswordQuestionVisible);
				fs.First(bf => bf.DataField == "Comment").Visible = Extensions.TryParseBool(Resources.Default.SecurityPanel_CommentVisible);
				fs.First(bf => bf.DataField == "UserDisplayName").Visible = Extensions.TryParseBool(Resources.Default.SecurityPanel_UserDisplayNameVisible);
			});
	}

	IEnumerable<GridView> GetNamedItemGridViews()
	{
		yield return this.userView;
		yield return this.roleView;
	}

	void OnViewRowEditing(object sender, GridViewEditEventArgs e)
	{
		this.BeginEdit((GridView)sender, e.NewEditIndex);
	}

	void OnViewRowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
	{
		this.CancelEditAll();
	}

	void OnViewRowUpdating(object sender, GridViewUpdateEventArgs e)
	{
		this.EndEdit((GridView)sender);
	}

	void OnViewRowDeleting(object sender, GridViewDeleteEventArgs e)
	{
		this.CancelEditAll();

		var gridView = (GridView)sender;

		if (gridView == this.roleView && this.userView.DataSource != null)
			foreach (var gridUser in this.userView.DataSource.To<IEnumerable>().Cast<GridUser>())
				gridUser.Roles = gridUser.Roles.Where(r => r != gridView.DataSource.To<GridNamedItem[]>()[e.RowIndex].Name).ToList();

		gridView.DataSource = gridView.DataSource.To<GridNamedItem[]>().Where((gu, index) => index != e.RowIndex).ToArray();
		this.GetNamedItemGridViews().ForEach(gv => gv.DataBind());
	}

	void OnNameValidatorServerValidate(object sender, ServerValidateEventArgs e)
	{
		var gridView = WebExtensions.FindAncestor<GridView>((Control)sender);
		var name = (e.Value == string.Empty ? null : e.Value);
		e.IsValid = !gridView.DataSource.To<GridNamedItem[]>().Where((gi, index) => index != gridView.EditIndex && (string.Equals(gi.Name, name, StringComparison.InvariantCultureIgnoreCase) || string.Equals(gi.OriginalName, name, StringComparison.InvariantCultureIgnoreCase))).Any();
	}

	void OnNewUserButtonClick(object sender, EventArgs e)
	{
		var newUser = new GridUser { Roles = new List<string>() };
		this.EditNewItem(this.userView, newUser);
	}

	void OnNewRoleButtonClick(object sender, EventArgs e)
	{
		var newRole = new GridRole { PermissionEntries = new List<PermissionEntry>() };
		this.EditNewItem(this.roleView, newRole);
	}

	void OnUserValidatorServerValidate(object sender, ServerValidateEventArgs e)
	{
		var validator = (CustomValidator)sender;

		if (!this.ValidateGrid(this.userView))
		{
			e.IsValid = false;
			validator.Text = Resources.Default.SecurityPanel_MustCompleteErrorMessage;
		}

		var administerRoleNames = this.roleView.DataSource.To<IEnumerable>().Cast<GridRole>().Where(gr => gr.PermissionEntries.Any(pe => pe.Name == PermissionInfo.AdministerPermission)).Select(r => r.Name);

		if (e.IsValid && !this.userView.DataSource.To<IEnumerable>().Cast<GridUser>().Any(u => u.Roles.Intersect(administerRoleNames).Any()))
		{
			e.IsValid = false;
			validator.Text = Resources.Default.SecurityPanel_MustDefineAdministerUser;
		}
	}

	void OnRoleValidatorServerValidate(object sender, ServerValidateEventArgs e)
	{
		var validator = (CustomValidator)sender;

		if (!this.ValidateGrid(this.roleView))
		{
			e.IsValid = false;
			validator.Text = Resources.Default.SecurityPanel_MustCompleteErrorMessage;
		}

		if (e.IsValid && !this.roleView.DataSource.To<IEnumerable>().Cast<GridRole>().Any(gr => gr.PermissionEntries.Any(pe => pe.Name == PermissionInfo.AdministerPermission)))
		{
			e.IsValid = false;
			validator.Text = Resources.Default.SecurityPanel_MustDefineAdministerRole;
		}
	}

	void OnRepeaterItemCommand(object sender, RepeaterCommandEventArgs e)
	{
		if (e.CommandName == "Delete")
		{
			var repeater = (RepeaterEx)sender;
			repeater.DataSource.To<IList>().RemoveAt(e.Item.ItemIndex);
			repeater.Rebind();

			if (repeater.FindAncestor<GridView>() == this.userView)
			{
				var gridViewRow = repeater.FindAncestor<GridViewRow>();
				gridViewRow.FindControl("rolePanel").DataBind();
			}
		}
	}

	void OnPermissionListChanged(object sender, EventArgs e)
	{
		this.SetPermissionControlVisibility((Control)sender);
	}

	void OnPermissionPanelDataBinding(object sender, EventArgs e)
	{
		var panel = (Control)sender;

		using (var sessionManager = ServiceChannelPool<ISessionManagerChannel>.Instance.Borrow())
		{
			this.PopulateStringListItems(panel, "permissionList", PermissionInfo.GetPermissionNames());
			this.PopulateStringListItems(panel, "sessionGroupFilterList", Enum.GetNames(typeof(SessionGroupFilter)));
			this.PopulateStringListItems(panel, "sessionGroupList", sessionManager.GetSessionGroups().Select(sg => sg.Name));
			this.PopulateStringListItems(panel, "ownershipFilterList", Enum.GetNames(typeof(OwnershipFilter)));
		}

		this.SetPermissionControlVisibility(panel);
	}

	void OnRolePanelDataBinding(object sender, EventArgs e)
	{
		var panel = (Control)sender;
		var editingRoles = (IEnumerable<string>)panel.FindAncestor<GridViewRow>().GetAllControls().OfType<RepeaterEx>().First().DataSource;
		panel.Visible = this.PopulateStringListItems(panel, "roleList", ((GridNamedItem[])this.roleView.DataSource).Where(r => !editingRoles.Contains(r.Name)).Select(r => r.Name)) != 0;
	}

	void OnAddButtonClick(object sender, EventArgs e)
	{
		var addButton = (Control)sender;
		var gridViewRow = addButton.FindAncestor<GridViewRow>();
		var gridView = gridViewRow.FindAncestor<GridView>();
		var item = gridView.DataSource.To<GridNamedItem[]>()[gridView.EditIndex];

		if (gridView == this.userView)
		{
			var roleName = ((DropDownList)addButton.FindControl("roleList")).SelectedValue;
			item.As<GridUser>().EditingRoles.Add(roleName);
			gridViewRow.FindControl("rolePanel").DataBind();
		}

		if (gridView == this.roleView)
		{
			var permissionName = ((DropDownList)addButton.FindControl("permissionList")).SelectedValue;
			var sessionGroupFilter = Extensions.TryParseEnum<SessionGroupFilter>(addButton.FindControl("sessionGroupFilterList").To<DropDownList>().SelectedValue);
			var ownershipFilter = Extensions.TryParseEnum<OwnershipFilter>(addButton.FindControl("ownershipFilterList").To<DropDownList>().SelectedValue);
			var sessionGroupName = (sessionGroupFilter != SessionGroupFilter.SpecificSessionGroup ? null : addButton.FindControl("sessionGroupList").To<DropDownList>().SelectedValue);
			var permissionEntry = PermissionInfo.CreatePermissionEntry(permissionName, sessionGroupFilter, ownershipFilter, sessionGroupName);
			item.As<GridRole>().EditingPermissionEntries.Add(permissionEntry);
		}

		gridViewRow.GetAllControls().OfType<RepeaterEx>().First().Rebind();
	}

	void OnUserSourceBoxSelectedIndexChanged(object sender, EventArgs e)
	{
		var membershipSection = WebConfigurationManager.OpenWebConfiguration().GetSection<MembershipSection>();

		var gridRoleNames = this.roleView.DataSource
			.To<GridNamedItem[]>()
			.Select(gr => gr.Name)
			.ToArray();

		var membershipProvider = membershipSection.Providers
			.OfType<ProviderSettings>()
			.FirstOrDefault(p => p.Name == this.userSourceBox.SelectedValue)
			.SafeNav(p => WebConfigurationManager.TryCreateMembershipProvider(p.Type, p.Name, p.Parameters));

		this.BindUserSourceConfiguration(membershipProvider);
		this.BindFormsUsers(membershipProvider, gridRoleNames);
	}

	bool ValidateGrid(GridView gridView)
	{
		if (gridView.EditIndex == -1)
			return true;

		this.Page.Validate("Grid");

		if (!this.Page.IsValid)
			return false;

		this.EndEdit(gridView);
		return true;
	}

	int PopulateStringListItems(Control baseControl, string controlName, IEnumerable<string> items)
	{
		var control = (ListControl)baseControl.FindControl(controlName);
		control.Items.Clear();
		items.ForEach(item => control.Items.Add(item));
		return control.Items.Count;
	}

	void SetPermissionControlVisibility(Control control)
	{
		bool sessionGroupFilterRelevant, ownershipFilterRelevant, sessionGroupNameRelevant;
		var permissionList = (DropDownList)control.FindControl("permissionList");
		var sessionGroupFilterList = (DropDownList)control.FindControl("sessionGroupFilterList");
		var sessionGroupFilter = Extensions.TryParseEnum<SessionGroupFilter>(sessionGroupFilterList.SelectedValue);
		PermissionInfo.CalculateFieldRelevance(permissionList.SelectedValue, sessionGroupFilter, string.Empty, out sessionGroupFilterRelevant, out ownershipFilterRelevant, out sessionGroupNameRelevant);
		sessionGroupFilterList.Visible = sessionGroupFilterRelevant;
		control.FindControl("sessionGroupList").Visible = sessionGroupNameRelevant;
		control.FindControl("ownershipFilterList").Visible = ownershipFilterRelevant;
	}

	void EditNewItem(GridView gridView, GridNamedItem item)
	{
		var items = (GridNamedItem[])gridView.DataSource;
		gridView.DataSource = items.Concat(new GridNamedItem[] { item }).ToArray();
		this.BeginEdit(gridView, items.Length);
	}

	void BeginEdit(GridView gridView, int index)
	{
		this.CancelEditAll();

		var item = gridView.DataSource.To<IList>()[index];

		item.As<GridUser>().SafeDo(u => u.EditingRoles = u.Roles.ToList());
		item.As<GridRole>().SafeDo(r => r.EditingPermissionEntries = r.PermissionEntries.ToList());

		gridView.EditIndex = index;
		this.GetNamedItemGridViews().ForEach(gv => gv.DataBind());
	}

	void EndEdit(GridView gridView)
	{
		var items = (GridNamedItem[])gridView.DataSource;
		var item = items[gridView.EditIndex];
		var row = gridView.Rows[gridView.EditIndex];

		var oldName = item.Name;
		item.Name = this.GetCellEditText(row, 1);

		item.As<GridUser>().SafeDo(u =>
		{
			u.UserDisplayName = this.GetCellEditText(row, 2);
			u.Password = this.GetCellEditText(row, 3);
			u.PasswordQuestion = this.GetCellEditText(row, 4);
			u.Comment = this.GetCellEditText(row, 5);
			u.Roles = u.EditingRoles;
		});

		item.As<GridRole>().SafeDo(r =>
		{
			r.PermissionEntries = r.EditingPermissionEntries;

			if (oldName != r.Name)
			{
				foreach (var gridUser in this.userView.DataSource.To<IEnumerable>().SafeEnumerate().Cast<GridUser>())
				{
					var roleIndex = gridUser.Roles.IndexOf(oldName);

					if (roleIndex != -1)
					{
						gridUser.Roles.RemoveAt(roleIndex);
						gridUser.Roles.Insert(roleIndex, r.Name);
					}
				}
			}
		});

		gridView.EditIndex = -1;
		this.GetNamedItemGridViews().ForEach(gv => gv.DataBind());
	}

	string GetCellEditText(GridViewRow row, int cellIndex)
	{
		return row.Cells[cellIndex].Controls.OfType<TextBox>().First().GetTrimmedOrNullText();
	}

	void CancelEditAll()
	{
		this.GetNamedItemGridViews().ForEach(gv => this.TryCancelEdit(gv));
		this.GetNamedItemGridViews().ForEach(gv => gv.DataBind());
	}

	void TryCancelEdit(GridView gridView)
	{
		if (gridView.EditIndex != -1)
		{
			var items = (GridNamedItem[])gridView.DataSource;

			if (items[gridView.EditIndex].OriginalName == null)
			{
				gridView.DataSource = items.Where((gu, index) => index != gridView.EditIndex).ToArray();
			}
			else
			{
				var item = gridView.DataSource.To<IList>()[gridView.EditIndex];

				item.As<GridUser>().SafeDo(u => u.EditingRoles = null);
				item.As<GridRole>().SafeDo(u => u.EditingPermissionEntries = null);
			}

			gridView.EditIndex = -1;
		}
	}

	void OnUserLookupButtonClick(object sender, EventArgs e)
	{
		var resultBuilder = new StringBuilder();
		var membershipSection = WebConfigurationManager.OpenWebConfiguration().GetSection<MembershipSection>();

		var membershipProvider = membershipSection.Providers
			.OfType<ProviderSettings>()
			.FirstOrDefault(p => p.Name == this.userSourceBox.SelectedValue)
			.SafeNav(p => Tuple.Create(p, WebConfigurationManager.TryCreateMembershipProvider(p.Type, p.Name, p.Parameters)))
			.SafeNav(t => WebConfigurationManager.TryCreateMembershipProvider(
				t.Item1.Type,
				t.Item1.Name, 
				new NameValueCollection(t.Item1.Parameters).Mutate(c => this.SaveUserSourceConfiguration(t.Item2, c))
			));

		var currentRoleSet = this.roleView.DataSource
			.To<IEnumerable>()
			.Cast<GridNamedItem>()
			.Select(gni => gni.Name)
			.SafeNav(rns => new HashSet<string>(rns, StringComparer.InvariantCultureIgnoreCase));

		var userName = this.testUserNameBox.Text.Trim();

		try
		{
			var user = membershipProvider.GetUser(userName);

			if (user == null)
			{
				resultBuilder.Append(DR.SecurityPanel_UserLookupResult_NotFoundMessage);
			}
			else
			{
				var roleTexts = membershipProvider
					.As<IRoleMembershipProvider>()
					.SafeNav(rmp => rmp.GetRolesForUser(userName, false))
					.SafeEnumerate()
					.Select(rn => Tuple.Create(rn, currentRoleSet.Contains(rn)))
					.OrderBy(t => !t.Item2)
					.Select(t => string.Format(t.Item2 ? DR.SecurityPanel_UserLookupResult_RoleHasMatchesFormat : DR.SecurityPanel_UserLookupResult_RoleHasNoMatchesFormat, t.Item1))
					.ToArray();

				new[] {
				Tuple.Create(DR.SecurityPanel_UserLookupResult_UserNameLabelText, new[] { user.UserName }),
				Tuple.Create(DR.SecurityPanel_UserLookupResult_UserDisplayNameLabelText, new[] { user.As<IDisplayNameUser>().SafeNav(u => u.UserDisplayName) }),
				Tuple.Create(DR.SecurityPanel_UserLookupResult_EmailLabelText, new[] { user.Email }),
				Tuple.Create(DR.SecurityPanel_UserLookupResult_CommentLabelText, new[] { user.Comment }),
				Tuple.Create(DR.SecurityPanel_UserLookupResult_PasswordQuestionLabelText, new[] { user.PasswordQuestion }),
				Tuple.Create(DR.SecurityPanel_UserLookupResult_RolesLabelText, roleTexts),
			}.ForEach(t =>
				{
					resultBuilder.AppendLine(t.Item1);
					t.Item2.ForEach(i => resultBuilder.AppendLine('\t' + i));
					resultBuilder.AppendLine();
				});
			}
		}
		catch (Exception ex)
		{
			resultBuilder.AppendLine(ex.Message);
		}

		var script = string.Format(JavaScriptFormatter.Default, "SC.ui.showModalMessageBox({0:js}, {1:js});", DR.SecurityPanel_UserLookupResult_Title, resultBuilder.ToString());
		WebExtensions.RegisterUpdatePanelStartupScript(this, script);
	}

	void SetUserViewEmptyDataText()
	{
		this.userView.EmptyDataText = (this.userView.DataSource != null ? DR.SecurityPanel_EmptyUserMessage : DR.SecurityPanel_ExternalUsersMessage);
	}

	void BindFormsUsers(MembershipProvider membershipProvider, string[] allowedRoleNames)
	{
		try
		{
			this.userView.DataSource = membershipProvider.GetAllMembershipUsers().Select(m => new GridUser
			{
				OriginalName = m.UserName,
				Name = m.UserName,
				PasswordQuestion = m.PasswordQuestion,
				UserDisplayName = m.As<IDisplayNameUser>().SafeNav(d => d.UserDisplayName),
				Comment = m.Comment,
				Password = EmptyPassword,
				Roles = membershipProvider.As<IRoleMembershipProvider>().SafeNav(p => p.GetRolesForUser(m.UserName, false).Intersect(allowedRoleNames)).SafeEnumerate().ToList()
			}).OfType<GridNamedItem>().ToArray();
		}
		catch (Exception)
		{
			this.userView.DataSource = null;
		}

		this.SetUserViewEmptyDataText();
		this.userView.DataBind();
	}

	void BindUserSourceConfiguration(MembershipProvider membershipProvider)
	{
		this.userSourceConfigurationView.DataSource = membershipProvider
			.As<MembershipProviderBase>()
			.SafeNav(mp => mp.GetSettingsInfos())
			.SafeEnumerate()
			.Select(pi => new Pair(pi.Item1, membershipProvider.To<MembershipProviderBase>().GetSettingsValue(pi.Item1).SafeNav(o => o.ToString())))
			.ToArray();

		this.userSourceConfigurationView.DataBind();
	}

	void SaveUserSourceConfiguration(MembershipProvider membershipProvider, NameValueCollection parameters)
	{
		foreach (var row in this.userSourceConfigurationView.Rows.OfType<GridViewRow>())
		{
			var key = row.GetAllControls().OfType<Label>().First().Text;
			var value = row.GetAllControls().OfType<TextBox>().First().Text;
			membershipProvider.As<MembershipProviderBase>().SafeDo(mp => mp.SaveSettingToConfiguration(parameters, key, value));
		}
	}

	public void Refresh()
	{
		var membershipSection = WebConfigurationManager.OpenWebConfiguration().GetSection<MembershipSection>();

		membershipSection.Providers
			.OfType<ProviderSettings>()
			.Select(ps => new ListItem(
				DR.ResourceManager.GetString("SecurityPanel." + ps.Name + ".Text") ?? ps.Name,
				ps.Name
			))
			.ToArray()
			.SafeDo(lis =>
			{
				this.userSourceBox.Items.Clear();
				this.userSourceBox.Items.AddRange(lis);
				this.userSourceBox.SelectedValue = membershipSection.DefaultProvider;
			});

		var gridRoles = Permissions.Provider
			.GetAllRoles()
			.Select(r => new GridRole
			{
				OriginalName = r.Name,
				Name = r.Name,
				PermissionEntries = new List<PermissionEntry>(r.PermissionEntries)
			}).OfType<GridNamedItem>().ToArray();

		this.roleView.DataSource = gridRoles;
		this.roleView.DataBind();

		var membershipProvider = membershipSection.Providers
			.OfType<ProviderSettings>()
			.FirstOrDefault(p => p.Name == membershipSection.DefaultProvider)
			.SafeNav(p => WebConfigurationManager.TryCreateMembershipProvider(p.Type, p.Name, p.Parameters));

		this.BindUserSourceConfiguration(membershipProvider);
		this.BindFormsUsers(membershipProvider, gridRoles.Select(gr => gr.Name).ToArray());
	}

	public void Save()
	{
		var configuration = WebConfigurationManager.OpenWebConfiguration();
		var membershipSection = WebConfigurationManager.GetSection<MembershipSection>(configuration);

		var previousDefaultProvider = membershipSection.DefaultProvider;
		membershipSection.DefaultProvider = this.userSourceBox.SelectedValue;

		var selectedMembershipProviderSettings = membershipSection.Providers
			.OfType<ProviderSettings>()
			.Where(ps => ps.Name == membershipSection.DefaultProvider)
			.First();

		var dummyMembershipProvider = WebConfigurationManager.TryCreateMembershipProvider(selectedMembershipProviderSettings.Type, selectedMembershipProviderSettings.Name, selectedMembershipProviderSettings.Parameters);

		var previousSettingsString = Extensions.EncodeQueryString(selectedMembershipProviderSettings.Parameters);
		selectedMembershipProviderSettings.SafeDo(ps => this.SaveUserSourceConfiguration(dummyMembershipProvider, ps.Parameters));

		if (membershipSection.DefaultProvider != previousDefaultProvider || previousSettingsString != Extensions.EncodeQueryString(selectedMembershipProviderSettings.Parameters))
			ServerToolkit.Instance.SaveConfiguration(configuration);

		this.roleView.DataSource
			.To<IEnumerable>()
			.Cast<GridRole>()
			.Select(gr => new Role { Name = gr.Name, PermissionEntries = gr.PermissionEntries.ToArray() })
			.SafeDo(rs => Permissions.Provider.SaveRoles(rs));

		if (this.userView.DataSource != null)
		{
			var membershipProvider = WebConfigurationManager.TryCreateMembershipProvider(selectedMembershipProviderSettings.Type, selectedMembershipProviderSettings.Name, selectedMembershipProviderSettings.Parameters);
			var gridUsers = this.userView.DataSource.As<GridNamedItem[]>().Cast<GridUser>().ToArray();

			foreach (var existingMembershipUser in membershipProvider.GetAllMembershipUsers())
				if (!gridUsers.Any(u => u.OriginalName == existingMembershipUser.UserName))
					membershipProvider.DeleteUser(existingMembershipUser.UserName);

			foreach (var gridUser in gridUsers)
			{
				if (gridUser.OriginalName == null)
				{
					membershipProvider.CreateUser(gridUser.Name, gridUser.Password);
				}
				else
				{
					if (gridUser.OriginalName != gridUser.Name)
						if (!membershipProvider.To<IChangeUserNameProvider>().ChangeUserName(gridUser.OriginalName, gridUser.Name))
							throw new InvalidOperationException("Unable to change user name for: " + gridUser.OriginalName);

					if (gridUser.Password != EmptyPassword)
						if (!membershipProvider.ChangePassword(gridUser.Name, null, gridUser.Password))
							throw new InvalidOperationException("Unable to change password for: " + gridUser.Name);
				}

				if (!membershipProvider.ChangePasswordQuestionAndAnswer(gridUser.Name, null, gridUser.PasswordQuestion, null))
					throw new InvalidOperationException("Unable to change password question for: " + gridUser.Name);

				var user = membershipProvider.GetUser(gridUser.Name);
				user.As<IDisplayNameUser>().SafeDo(u => u.UserDisplayName = gridUser.UserDisplayName);
				user.Comment = gridUser.Comment;
				membershipProvider.UpdateUser(user);

				membershipProvider
					.As<IRoleMembershipProvider>()
					.SafeDo(rmp =>
					{
						var userRoles = rmp.GetRolesForUser(gridUser.Name, false);

						if (userRoles.Length != gridUser.Roles.Count || userRoles.Intersect(gridUser.Roles).Count() != userRoles.Length)
						{
							rmp.RemoveUsersFromRoles(new[] { gridUser.Name }, userRoles);
							rmp.AddUsersToRoles(new[] { gridUser.Name }, gridUser.Roles.ToArray());
						}
					});
			}
		}
	}

	public bool IsSaveAllowed { get { return true; } }

</script>
<dl class="SecurityPanel">
	<dt runat="server" innerhtml="<%$ Resources:Default, SecurityPanel.UserSourceLabelText %>" />
	<dd>
		<asp:DropDownList runat="server" ID="userSourceBox" AutoPostBack="true" OnSelectedIndexChanged="OnUserSourceBoxSelectedIndexChanged" />
	</dd>
	<dt runat="server" innerhtml="<%$ Resources:Default, SecurityPanel.UserSourceConfigurationLabelText %>" />
	<dd>
		<asp:GridView runat="server" ID="userSourceConfigurationView" CssClass="DataTable" AutoGenerateColumns="false" EmptyDataText="<%$ Resources:Default, SecurityPanel.EmptyUserSourceConfigurationMessage %>">
			<Columns>
				<asp:TemplateField HeaderText="<%$ Resources:Default, SecurityPanel.KeyHeaderText %>">
					<ItemTemplate>
						<asp:Label runat="server" Text="<%# Container.DataItem.As<Pair>().First %>" />
					</ItemTemplate>
				</asp:TemplateField>
				<asp:TemplateField HeaderText="<%$ Resources:Default, SecurityPanel.ValueHeaderText %>">
					<ItemTemplate>
						<asp:TextBox runat="server" Text="<%# Container.DataItem.As<Pair>().Second %>" />
					</ItemTemplate>
				</asp:TemplateField>
			</Columns>
		</asp:GridView>
	</dd>
	<dt runat="server" innerhtml="<%$ Resources:Default, SecurityPanel.UsersLabelText %>" />
	<dd>
		<asp:GridView runat="server" ID="userView" EnableViewState="false" CssClass="DataTable" AutoGenerateColumns="false" OnRowEditing="OnViewRowEditing" OnRowUpdating="OnViewRowUpdating" OnRowCancelingEdit="OnViewRowCancelingEdit" OnRowDeleting="OnViewRowDeleting">
			<Columns>
				<asp:TemplateField>
					<EditItemTemplate>
						<asp:LinkButton runat="server" CommandName="Update" Text="<%$ Resources:Default, SecurityPanel.DoneButtonText %>" CausesValidation="true" ValidationGroup="Grid" />
						<asp:LinkButton runat="server" CommandName="Cancel" Text="<%$ Resources:Default, SecurityPanel.CancelButtonText %>" CausesValidation="false" />
					</EditItemTemplate>
					<ItemTemplate>
						<asp:LinkButton runat="server" CommandName="Edit" Text="<%$ Resources:Default, SecurityPanel.EditButtonText %>" CausesValidation="false" Enabled="<%# this.userView.EditIndex == -1 && this.roleView.EditIndex == -1 %>" />
						<asp:LinkButton runat="server" CommandName="Delete" Text="<%$ Resources:Default, SecurityPanel.DeleteButtonText %>" CausesValidation="false" Enabled="<%# this.userView.EditIndex == -1 && this.roleView.EditIndex == -1 %>" />
					</ItemTemplate>
				</asp:TemplateField>
				<asp:TemplateField HeaderText="<%$ Resources:Default, SecurityPanel.UserNameHeaderText %>">
					<ItemTemplate>
						<%# Eval("Name") %>
					</ItemTemplate>
					<EditItemTemplate>
						<asp:TextBox runat="server" ID="NameBox" Text='<%# Eval("Name") %>' />
						<asp:RequiredFieldValidator runat="server" ControlToValidate="NameBox" Text="*" EnableClientScript="false" ValidationGroup="Grid" Display="Dynamic" />
						<asp:CustomValidator runat="server" ControlToValidate="NameBox" Text="*" ValidationGroup="Grid" OnServerValidate="OnNameValidatorServerValidate" Display="Dynamic" />
					</EditItemTemplate>
				</asp:TemplateField>
				<asp:BoundField DataField="UserDisplayName" HeaderText="<%$ Resources:Default, SecurityPanel.UserDisplayNameHeaderText %>" />
				<asp:TemplateField HeaderText="<%$ Resources:Default, SecurityPanel.PasswordHeaderText %>">
					<ItemTemplate>
						<%= new string('*', EmptyPassword.Length) %>
					</ItemTemplate>
					<EditItemTemplate>
						<asp:Literal runat="server" Text="<%$ Resources:Default, SecurityPanel.EnterPasswordLabelText %>" />
						<asp:PasswordTextBox runat="server" ID="PasswordBox" Text='<%# Eval("Password") %>' autocomplete="off" />
						<asp:RequiredFieldValidator runat="server" ControlToValidate="PasswordBox" Text="*" EnableClientScript="false" ValidationGroup="Grid" Display="Dynamic" /><br />
						<asp:Literal runat="server" Text="<%$ Resources:Default, SecurityPanel.VerifyPasswordLabelText %>" />
						<asp:PasswordTextBox runat="server" ID="VerifyPasswordBox" Text='<%# Eval("Password") %>' autocomplete="off" />
						<asp:CompareValidator runat="server" ControlToValidate="PasswordBox" ControlToCompare="VerifyPasswordBox" Text="*" EnableClientScript="false" ValidationGroup="Grid" Display="Dynamic" />
					</EditItemTemplate>
				</asp:TemplateField>
				<asp:BoundField DataField="PasswordQuestion" HeaderText="<%$ Resources:Default, SecurityPanel.PasswordQuestionHeaderText %>" />
				<asp:BoundField DataField="Comment" HeaderText="<%$ Resources:Default, SecurityPanel.CommentHeaderText %>" />
				<asp:TemplateField HeaderText="<%$ Resources:Default, SecurityPanel.RolesHeaderText %>">
					<ItemTemplate>
						<asp:RepeaterEx runat="server" DataSource='<%# Eval("Roles") %>' Delimiter="LineBreak" />
					</ItemTemplate>
					<EditItemTemplate>
						<asp:RepeaterEx runat="server" DataSource='<%# Eval("EditingRoles") %>' OnItemCommand="OnRepeaterItemCommand" Delimiter="LineBreak">
							<ItemTemplate>
								<%# Container.DataItem %>
								<asp:LinkButton runat="server" CommandName="Delete" Text="[X]" CausesValidation="false" />
							</ItemTemplate>
						</asp:RepeaterEx>
						<div runat="server" id="rolePanel" ondatabinding="OnRolePanelDataBinding">
							<asp:DropDownList runat="server" ID="roleList" />
							<asp:LinkButton runat="server" Text="<%$ Resources:Default, SecurityPanel.AddButtonText %>" OnClick="OnAddButtonClick" CausesValidation="false" />
						</div>
					</EditItemTemplate>
				</asp:TemplateField>
			</Columns>
		</asp:GridView>
		<asp:LinkButton runat="server" ID="newUserButton" Text="<%$ Resources:Default, SecurityPanel.NewUserButtonText %>" OnClick="OnNewUserButtonClick" CausesValidation="false" />
		<asp:CustomValidator runat="server" ID="userValidator" EnableClientScript="false" OnServerValidate="OnUserValidatorServerValidate" />
	</dd>
	<dt runat="server" innerhtml="<%$ Resources:Default, SecurityPanel.UserTestLabelText %>" />
	<dd>
		<asp:TextBox runat="server" ID="testUserNameBox" />
		<asp:Button runat="server" ID="testButton" Text="<%$ Resources:Default, SecurityPanel.LookupUserButtonText %>" OnClick="OnUserLookupButtonClick" CausesValidation="false" />
		<asp:LoadingImage runat="server" CssClass="SmallLoadingImage" PostBackControl="testButton" ImageUrl="Images/ActivityIndicator.gif" />
	</dd>
	<dt runat="server" innerhtml="<%$ Resources:Default, SecurityPanel.RolesLabelText %>" />
	<dd>
		<asp:GridView runat="server" ID="roleView" EnableViewState="false" CssClass="DataTable" EmptyDataText="<%$ Resources:Default, SecurityPanel.EmptyRoleMessage %>" AutoGenerateColumns="false" OnRowEditing="OnViewRowEditing" OnRowUpdating="OnViewRowUpdating" OnRowCancelingEdit="OnViewRowCancelingEdit" OnRowDeleting="OnViewRowDeleting">
			<Columns>
				<asp:TemplateField>
					<EditItemTemplate>
						<asp:LinkButton runat="server" CommandName="Update" Text="<%$ Resources:Default, SecurityPanel.DoneButtonText %>" CausesValidation="true" ValidationGroup="Grid" />
						<asp:LinkButton runat="server" CommandName="Cancel" Text="<%$ Resources:Default, SecurityPanel.CancelButtonText %>" CausesValidation="false" />
					</EditItemTemplate>
					<ItemTemplate>
						<asp:LinkButton runat="server" CommandName="Edit" Text="<%$ Resources:Default, SecurityPanel.EditButtonText %>" CausesValidation="false" Enabled="<%# this.userView.EditIndex == -1 && this.roleView.EditIndex == -1 %>" />
						<asp:LinkButton runat="server" CommandName="Delete" Text="<%$ Resources:Default, SecurityPanel.DeleteButtonText %>" CausesValidation="false" Enabled="<%# this.userView.EditIndex == -1 && this.roleView.EditIndex == -1 %>" />
					</ItemTemplate>
				</asp:TemplateField>
				<asp:TemplateField HeaderText="<%$ Resources:Default, SecurityPanel.RoleNameHeaderText %>">
					<ItemTemplate>
						<%# Eval("Name") %>
					</ItemTemplate>
					<EditItemTemplate>
						<asp:TextBox runat="server" ID="NameBox" Text='<%# Eval("Name") %>' />
						<asp:RequiredFieldValidator runat="server" ControlToValidate="NameBox" Text="*" EnableClientScript="false" ValidationGroup="Grid" Display="Dynamic" />
						<asp:CustomValidator runat="server" ControlToValidate="NameBox" Text="*" ValidationGroup="Grid" OnServerValidate="OnNameValidatorServerValidate" Display="Dynamic" />
					</EditItemTemplate>
				</asp:TemplateField>
				<asp:TemplateField HeaderText="<%$ Resources:Default, SecurityPanel.PermissionsHeaderText %>">
					<ItemTemplate>
						<asp:RepeaterEx runat="server" DataSource='<%# Eval("PermissionEntries") %>' Delimiter="LineBreak" />
					</ItemTemplate>
					<EditItemTemplate>
						<asp:RepeaterEx runat="server" DataSource='<%# ((GridRole)Container.DataItem).EditingPermissionEntries %>' OnItemCommand="OnRepeaterItemCommand" Delimiter="LineBreak">
							<ItemTemplate>
								<%# Container.DataItem %>
								<asp:LinkButton runat="server" CommandName="Delete" Text="[X]" CausesValidation="false" />
							</ItemTemplate>
						</asp:RepeaterEx>
						<div runat="server" ondatabinding="OnPermissionPanelDataBinding">
							<asp:DropDownList runat="server" ID="permissionList" AutoPostBack="true" OnSelectedIndexChanged="OnPermissionListChanged" />
							<asp:DropDownList runat="server" ID="sessionGroupFilterList" AutoPostBack="true" OnSelectedIndexChanged="OnPermissionListChanged" />
							<asp:DropDownList runat="server" ID="sessionGroupList" />
							<asp:DropDownList runat="server" ID="ownershipFilterList" />
							<asp:LinkButton runat="server" Text="<%$ Resources:Default, SecurityPanel.AddButtonText %>" OnClick="OnAddButtonClick" CausesValidation="false" />
						</div>
					</EditItemTemplate>
				</asp:TemplateField>
			</Columns>
		</asp:GridView>
		<asp:LinkButton runat="server" ID="newRoleButton" Text="<%$ Resources:Default, SecurityPanel.NewRoleButtonText %>" OnClick="OnNewRoleButtonClick" CausesValidation="false" />
		<asp:CustomValidator runat="server" ID="roleValidator" EnableClientScript="false" OnServerValidate="OnRoleValidatorServerValidate" />
	</dd>
</dl>
