﻿<%@ Control Language="C#" %>
<%@ Implements Interface="Elsinore.ScreenConnect.IDataControl" %>
<script runat="server">

	protected override void LoadViewState(object savedState)
	{
		var mainState = (Pair)savedState;
		base.LoadViewState(mainState.First);

		this.triggerList.DataSource = mainState.Second;
		this.triggerList.DataBind();
	}

	protected override object SaveViewState()
	{
		return new Pair(
			base.SaveViewState(),
			this.triggerList.DataSource
		);
	}

	public void Refresh()
	{
		using (var sessionManager = ServiceChannelPool<ISessionManagerChannel>.Instance.Borrow())
		{
			this.triggerList.DataSource = sessionManager.GetSessionEventTriggers();
			this.triggerList.DataBind();
		}
	}

	public void Save()
	{
		this.SaveToDataSource();

		using (var sessionManager = ServiceChannelPool<ISessionManagerChannel>.Instance.Borrow())
		{
			var sessionEventTriggers = (SessionEventTrigger[])this.triggerList.DataSource;
			sessionManager.SaveSessionEventTriggers(sessionEventTriggers);
		}
	}

	void SaveToDataSource()
	{
		foreach (var repeater in this.GetAllControls().OfType<Repeater>())
			repeater.SaveTextControls();
	}

	public bool IsSaveAllowed { get { return true; } }

	void OnTriggerListItemCommand(object sender, RepeaterCommandEventArgs e)
	{
		this.SaveToDataSource();
		
		if (e.CommandName == "AddAction")
		{
			var trigger = ((IList<SessionEventTrigger>)this.triggerList.DataSource)[e.Item.ItemIndex];
			trigger.Actions = trigger.Actions.SafeEnumerate().Append((string)e.CommandArgument == "Smtp" ? (TriggerAction)new SmtpTriggerAction() : new HttpTriggerAction()).ToArray();
		}
		else if (e.CommandName == "Move")
		{
			var triggers = (SessionEventTrigger[])this.triggerList.DataSource;
			Elsinore.ScreenConnect.Extensions.Swap(ref triggers[e.Item.ItemIndex], ref triggers[e.Item.ItemIndex + int.Parse((string)e.CommandArgument)]);
		}
		else if (e.CommandName == "Delete")
		{
			this.triggerList.DataSource = ((SessionEventTrigger[])this.triggerList.DataSource).Where((t, i) => i != e.Item.ItemIndex).ToArray();
		}

		this.triggerList.DataBind();
	}

	void OnActionListItemCommand(object sender, RepeaterCommandEventArgs e)
	{
		var actionRepeater = e.Item.FindAncestor<Repeater>();
		var triggerItem = actionRepeater.FindAncestor<RepeaterItem>();

		this.SaveToDataSource();
		
		var trigger = ((IList<SessionEventTrigger>)this.triggerList.DataSource)[triggerItem.ItemIndex];
		trigger.Actions = trigger.Actions.Where((a, i) => i != e.Item.ItemIndex).ToArray();

		this.triggerList.DataBind();
	}

	protected override bool OnBubbleEvent(object source, EventArgs e)
	{
		if (e is CommandEventArgs)
		{
			this.SaveToDataSource();
			this.triggerList.DataSource = ((SessionEventTrigger[])this.triggerList.DataSource).Append(new SessionEventTrigger()).ToArray();
			this.triggerList.DataBind();
			return true;
		}

		return base.OnBubbleEvent(source, e);
	}

</script>
<dl>
	<dt runat="server" innerhtml="<%$ Resources:Default, TriggersPanel.SessionEventTriggersText %>" />
	<dd>
		<table id="sessionEventTriggerTable" class="DataTable" cellspacing="0">
			<asp:RepeaterEx runat="server" ID="triggerList" OnItemCommand="OnTriggerListItemCommand" AutoBindTextControls="true" EnableViewState="false">
				<HeaderTemplate>
					<tr>
						<th style="width: 12%">&nbsp;</th>
						<th style="width: 88%"><%= DR.TriggersPanel_TriggerHeader %></th>
					</tr>
				</HeaderTemplate>
				<ItemTemplate>
					<tr>
						<td class="ButtonCell">
							<asp:LinkButtonEx runat="server" CommandName="Move" CommandArgument="-1" Text='<%$ Resources:Default, TriggersPanel.MoveUpButtonText %>' Enabled='<%# Container.ItemIndex != 0 %>' CssClass='<%# Container.ItemIndex == 0 ? "Disabled" : null %>' />
							<asp:LinkButtonEx runat="server" CommandName="Move" CommandArgument="1" Text='<%$ Resources:Default, TriggersPanel.MoveDownButtonText %>' Enabled='<%# Container.ItemIndex != ((IList)((Repeater)Container.Parent).DataSource).Count - 1 %>' />
							<asp:LinkButtonEx runat="server" CommandName="Delete" Text='<%$ Resources:Default, TriggersPanel.DeleteButtonText %>' />
						</td>
						<td>
							<dl>
								<dt><%= DR.TriggersPanel_TriggerNameLabelText %></dt>
								<dd>
									<asp:TextBox runat="server" ID="Name" />
								</dd>
								<dt><%= DR.TriggersPanel_TriggerEventFilterLabelText %></dt>
								<dd>
									<asp:TextBox runat="server" ID="EventFilter" TextMode="MultiLine" />
								</dd>
								<asp:TriggerActionRepeater runat="server" DataSource='<%# Eval("Actions") %>' OnItemCommand="OnActionListItemCommand" AutoBindTextControls="true">
									<ItemTemplate>
										<dt>
											<%# string.Format(DR.TriggersPanel_TriggerActionLabelFormat, Container.ItemIndex + 1) %>
											<br />
											<asp:LinkButton runat="server" CommandName="RemoveAction" Text='<%$ Resources:Default, TriggersPanel.RemoveActionButtonText %>' />
										</dt>
									</ItemTemplate>
									<SmtpTriggerActionItemTemplate>
										<dd>
											<dl>
												<dt><%# DR.TriggersPanel_MailActionFromLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="From" placeholder="<%# DR.TriggersPanel_MailActionFromPlaceHolder %>" /></dd>
												<dt><%# DR.TriggersPanel_MailActionToLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="To" placeholder="<%# DR.TriggersPanel_MailActionToPlaceHolder %>" /></dd>
												<dt><%# DR.TriggersPanel_MailActionSubjectLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="Subject" /></dd>
												<dt><%# DR.TriggersPanel_MailActionIsBodyHtmlLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="IsBodyHtml" /></dd>
												<dt><%# DR.TriggersPanel_MailActionBodyLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="Body" TextMode="MultiLine" /></dd>
											</dl>
										</dd>
									</SmtpTriggerActionItemTemplate>
									<HttpTriggerActionItemTemplate>
										<dd>
											<dl>
												<dt><%# DR.TriggersPanel_HttpActionUriLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="Uri" /></dd>
												<dt><%# DR.TriggersPanel_HttpActionHttpMethodLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="HttpMethod" placeholder="<%# DR.TriggersPanel_HttpActionHttpMethodPlaceHolder %>" /></dd>
												<dt><%# DR.TriggersPanel_HttpActionContentTypeLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="ContentType" placeholder="<%# DR.TriggersPanel_HttpActionContentTypePlaceHolder %>" /></dd>
												<dt><%# DR.TriggersPanel_HttpActionBodyLabelText %></dt>
												<dd><asp:TextBox runat="server" ID="Body" TextMode="MultiLine" placeholder="<%# DR.TriggersPanel_HttpActionBodyPlaceHolder %>" /></dd>
											</dl>
										</dd>
									</HttpTriggerActionItemTemplate>
								</asp:TriggerActionRepeater>
								<dt>
									<%# string.Format(DR.TriggersPanel_TriggerActionLabelFormat, Container.GetAllControls().OfType<Repeater>().First().Items.Count + 1) %>
								</dt>
								<dd>
									<asp:LinkButton runat="server" CommandName="AddAction" CommandArgument="Smtp" Text='<%$ Resources:Default, TriggersPanel.NewSmtpActionButtonText %>' />
									<asp:LinkButton runat="server" CommandName="AddAction" CommandArgument="Http" Text='<%$ Resources:Default, TriggersPanel.NewHttpActionButtonText %>' />
								</dd>
							</dl>
						</td>
					</tr>
				</ItemTemplate>
			</asp:RepeaterEx>
		</table>
		<p>
			<asp:LinkButton runat="server" CommandName="AddTrigger" Text='<%$ Resources:Default, TriggersPanel.NewTriggerButtonText %>' />
		</p>
	</dd>
</dl>
